/*
 *  ICARUS -  Interactive platform for Corpus Analysis and Research tools, University of Stuttgart
 *  Copyright (C) 2012-2013 Markus Gärtner and Gregor Thiele
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses.
 *
 * $Revision: 297 $
 * $Date: 2014-09-18 01:17:59 +0200 (Do, 18 Sep 2014) $
 * $URL: https://subversion.assembla.com/svn/icarusplatform/trunk/Icarus/core/de.ims.icarus/source/de/ims/icarus/ui/NumberDocument.java $
 *
 * $LastChangedDate: 2014-09-18 01:17:59 +0200 (Do, 18 Sep 2014) $
 * $LastChangedRevision: 297 $
 * $LastChangedBy: mcgaerty $
 */
package de.ims.icarus.ui;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;

public class NumberDocument extends PlainDocument {

	private static final long serialVersionUID = -7247806020719831949L;

	private static final String integerPattern = "^[+-]?\\d*$"; //$NON-NLS-1$
	private static final String decimalPattern = "^[+-]?\\d*\\.?\\d*$"; //$NON-NLS-1$

	private final String pattern;

	public NumberDocument() {
		this(false);
	}

	public NumberDocument(boolean allowFloatingPoint) {
		pattern = allowFloatingPoint ? decimalPattern : integerPattern;
	}

	@Override
	public void insertString(int offset, String str, AttributeSet a)
			throws BadLocationException {
		super.insertString(offset, str, a);
		String newText = getText(0, getLength());
		if (!newText.matches(pattern)) {
			remove(offset, str.length());
		}
	}
}