/*
 *  ICARUS -  Interactive platform for Corpus Analysis and Research tools, University of Stuttgart
 *  Copyright (C) 2012-2013 Markus Gärtner and Gregor Thiele
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses.
 *
 * $Revision: 123 $
 * $Date: 2013-07-31 17:22:01 +0200 (Mi, 31 Jul 2013) $
 * $URL: https://subversion.assembla.com/svn/icarusplatform/trunk/Icarus/core/de.ims.icarus/source/de/ims/icarus/ui/actions/BindableAction.java $
 *
 * $LastChangedDate: 2013-07-31 17:22:01 +0200 (Mi, 31 Jul 2013) $ 
 * $LastChangedRevision: 123 $ 
 * $LastChangedBy: mcgaerty $
 */
package de.ims.icarus.ui.actions;

import java.util.Map;

import javax.swing.Action;

/**
 * @author Markus Gärtner
 * @version $Id: BindableAction.java 123 2013-07-31 15:22:01Z mcgaerty $
 *
 */
public interface BindableAction extends Action {

	/**
	 * Binds this {@code Action} to the specified {@code owner}
	 * using the given collection of properties to configure it.
	 * Note that the returned {@code Action} does not have to be
	 * this very instance. There might be implementations that
	 * use a singleton class instance for performance or other
	 * reasons. In this case the signature of this method allows
	 * for proper configuration and the single shared instance
	 * can be returned (typically the object this method is called
	 * on will be discarded unless explicitly returned as bound
	 * action).
	 * 
	 * @param owner
	 * @param properties
	 * @return the fully configured and bound {@code Action}
	 */
	Action bind(Object owner, Map<String, Object> properties);
}
