/*
 *  ICARUS -  Interactive platform for Corpus Analysis and Research tools, University of Stuttgart
 *  Copyright (C) 2012-2013 Markus Gärtner and Gregor Thiele
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses.
 *
 * $Revision: 321 $
 * $Date: 2014-11-19 14:34:37 +0100 (Mi, 19 Nov 2014) $
 * $URL: https://subversion.assembla.com/svn/icarusplatform/trunk/Icarus/core/de.ims.icarus/source/de/ims/icarus/ui/list/ListUtils.java $
 *
 * $LastChangedDate: 2014-11-19 14:34:37 +0100 (Mi, 19 Nov 2014) $
 * $LastChangedRevision: 321 $
 * $LastChangedBy: mcgaerty $
 */
package de.ims.icarus.ui.list;

import java.util.ArrayList;
import java.util.List;

import javax.swing.JList;
import javax.swing.ListModel;
import javax.swing.ListSelectionModel;

/**
 * @author Markus Gärtner
 * @version $Id: ListUtils.java 321 2014-11-19 13:34:37Z mcgaerty $
 *
 */
public final class ListUtils {

	private ListUtils() {
		// no-op
	}

	public static <E extends Object> E getSelectedItem(JList<E> list) {
		if(list==null)
			throw new NullPointerException("Invalid list"); //$NON-NLS-1$

		ListModel<E> model = list.getModel();
		int selectedIndex = list.getSelectedIndex();

		return selectedIndex>-1 && selectedIndex<model.getSize() ?
				model.getElementAt(selectedIndex) : null;
	}

	public static <T extends Object> int indexOf(T item, ListModel<T> model) {
		if(model==null)
			throw new NullPointerException("Invalid list model"); //$NON-NLS-1$

		if(item==null)
			return -1;

		for(int i=0; i<model.getSize(); i++) {
			if(item.equals(model.getElementAt(i)))
				return i;
		}

		return -1;
	}

	public static <T extends Object> List<T> asList(ListModel<T> model) {
		List<T> result = new ArrayList<>();

		for(int i=0; i<model.getSize(); i++) {
			result.add(model.getElementAt(i));
		}

		return result;
	}

	public static void copySelectionState(ListSelectionModel m1, ListSelectionModel m2) {
		m2.setValueIsAdjusting(true);
		try {
			m2.clearSelection();

			if(m1.isSelectionEmpty())
				return;

			switch (m1.getSelectionMode()) {
			case ListSelectionModel.MULTIPLE_INTERVAL_SELECTION:
				for(int i=m1.getMinSelectionIndex(); i<=m1.getMaxSelectionIndex(); i++) {
					// TODO group intervals and apply selection on the target
				}
				break;

			default:
				m2.setSelectionInterval(m1.getMinSelectionIndex(), m1.getMaxSelectionIndex());
				break;
			}
		} finally {
			m2.setValueIsAdjusting(false);
		}
	}
}
