/*
 *  ICARUS -  Interactive platform for Corpus Analysis and Research tools, University of Stuttgart
 *  Copyright (C) 2012-2013 Markus Gärtner and Gregor Thiele
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses.
 *
 * $Revision: 329 $
 * $Date: 2014-12-13 13:57:36 +0100 (Sa, 13 Dez 2014) $
 * $URL: https://subversion.assembla.com/svn/icarusplatform/trunk/Icarus/core/de.ims.icarus/source/de/ims/icarus/ui/table/TablePresenter.java $
 *
 * $LastChangedDate: 2014-12-13 13:57:36 +0100 (Sa, 13 Dez 2014) $
 * $LastChangedRevision: 329 $
 * $LastChangedBy: mcgaerty $
 */
package de.ims.icarus.ui.table;

import java.awt.BorderLayout;
import java.awt.Component;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import de.ims.icarus.ui.UIUtil;
import de.ims.icarus.ui.actions.ActionComponentBuilder;
import de.ims.icarus.ui.view.AWTPresenter;
import de.ims.icarus.ui.view.PresenterUtils;
import de.ims.icarus.ui.view.UnsupportedPresentationDataException;
import de.ims.icarus.util.Options;
import de.ims.icarus.util.data.ContentType;
import de.ims.icarus.util.data.ContentTypeRegistry;


/**
 * @author Markus Gärtner
 * @version $Id: TablePresenter.java 329 2014-12-13 12:57:36Z mcgaerty $
 *
 */
public abstract class TablePresenter implements AWTPresenter.TableBasedPresenter {

	protected JTable table;

	protected JPanel contentPanel;

	protected TablePresenter() {
		init();
	}

	protected void init() {
		// for subclasses
	}

	protected abstract JTable createTable();

	protected ActionComponentBuilder createToolBar() {
		return null;
	}

	protected void buildPanel() {
		contentPanel = new JPanel(new BorderLayout());

		JTable table = getTable();
		JScrollPane scrollPane = new JScrollPane(table);
		scrollPane.setBorder(UIUtil.emptyBorder);
		contentPanel.add(scrollPane, BorderLayout.CENTER);


		ActionComponentBuilder builder = createToolBar();
		if(builder!=null) {
			contentPanel.add(builder.buildToolBar(), BorderLayout.NORTH);
		}
	}

	public JTable getTable() {
		if(table==null) {
			table = createTable();
		}

		return table;
	}

	protected abstract void setData(Object data, Options options);

	/**
	 * @see de.ims.icarus.ui.view.Presenter#present(java.lang.Object, de.ims.icarus.util.Options)
	 */
	@Override
	public void present(Object data, Options options)
			throws UnsupportedPresentationDataException {
		if(data==null)
			throw new NullPointerException("Invalid data"); //$NON-NLS-1$

		if(!PresenterUtils.presenterSupports(this, data))
			throw new UnsupportedPresentationDataException("Unsupported data: "+data.getClass()); //$NON-NLS-1$

		setData(data, options);

		if(table!=null) {
			table.revalidate();
			table.repaint();
		}
	}

	/**
	 * @see de.ims.icarus.ui.view.Presenter#close()
	 */
	@Override
	public void close() {
		// no-op
	}

	/**
	 * @see de.ims.icarus.ui.view.AWTPresenter#getPresentingComponent()
	 */
	@Override
	public Component getPresentingComponent() {
		if(contentPanel==null) {
			buildPanel();
		}
		return contentPanel;
	}

	/**
	 * @see de.ims.icarus.ui.view.Presenter#supports(de.ims.icarus.util.data.ContentType)
	 */
	@Override
	public boolean supports(ContentType type) {
		return ContentTypeRegistry.isCompatible(getContentType(), type);
	}

	protected abstract ContentType getContentType();
}
