/*
 *  ICARUS -  Interactive platform for Corpus Analysis and Research tools, University of Stuttgart
 *  Copyright (C) 2012-2013 Markus Gärtner and Gregor Thiele
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses.
 *
 * $Revision: 159 $
 * $Date: 2013-11-04 11:20:05 +0100 (Mo, 04 Nov 2013) $
 * $URL: https://subversion.assembla.com/svn/icarusplatform/trunk/Icarus/core/de.ims.icarus/source/de/ims/icarus/ui/tasks/Tasks.java $
 *
 * $LastChangedDate: 2013-11-04 11:20:05 +0100 (Mo, 04 Nov 2013) $ 
 * $LastChangedRevision: 159 $ 
 * $LastChangedBy: mcgaerty $
 */
package de.ims.icarus.ui.tasks;

import java.util.concurrent.Callable;

import javax.swing.SwingWorker;

import de.ims.icarus.util.WrapperException;


/**
 * @author Markus Gärtner
 * @version $Id: Tasks.java 159 2013-11-04 10:20:05Z mcgaerty $
 *
 */
public final class Tasks {

	private Tasks() {
		// no-op
	}
	
	public static SwingWorker<?, ?> createWorker(Runnable runnable) {
		if(runnable==null)
			throw new NullPointerException("Invalid runnable"); //$NON-NLS-1$
		
		return new RunnableWorker(runnable);
	}
	
	public static SwingWorker<?, ?> createWorker(Callable<?> callable) {
		if(callable==null)
			throw new NullPointerException("Invalid callable"); //$NON-NLS-1$
		
		return new CallableWorker(callable);
	}
	
	static class RunnableWorker extends SwingWorker<Object, Object> {
		
		private final Runnable runnable;

		public RunnableWorker(Runnable runnable) {
			this.runnable = runnable;
		}

		/**
		 * @see javax.swing.SwingWorker#doInBackground()
		 */
		@Override
		protected Object doInBackground() throws Exception {
			
			try {
				runnable.run();
			} catch(WrapperException e) {
				throw e.getWrappedException();
			}
			
			return null;
		}		
	}
	
	static class CallableWorker extends SwingWorker<Object, Object> {
		
		private final Callable<?> callable;

		public CallableWorker(Callable<?> callable) {
			this.callable = callable;
		}

		/**
		 * @see javax.swing.SwingWorker#doInBackground()
		 */
		@Override
		protected Object doInBackground() throws Exception {
			
			return callable.call();
		}		
	}
}
