/*
 *  ICARUS -  Interactive platform for Corpus Analysis and Research tools, University of Stuttgart
 *  Copyright (C) 2012-2013 Markus Gärtner and Gregor Thiele
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses.
 *
 * $Revision: 298 $
 * $Date: 2014-09-30 14:52:15 +0200 (Di, 30 Sep 2014) $
 * $URL: https://subversion.assembla.com/svn/icarusplatform/trunk/Icarus/core/de.ims.icarus/source/de/ims/icarus/ui/view/AWTPresenter.java $
 *
 * $LastChangedDate: 2014-09-30 14:52:15 +0200 (Di, 30 Sep 2014) $
 * $LastChangedRevision: 298 $
 * $LastChangedBy: mcgaerty $
 */
package de.ims.icarus.ui.view;

import java.awt.Component;

/**
 * Abstract {@code Presenter} that uses a {@code Component} to render
 * its presentation data. Implementations are not restricted to the
 * limitations of the AWT toolkit and may use Swing as well since there
 * is no special {@code Presenter} interface that explicitly enforces
 * the use of Swing components.
 *
 * @author Markus Gärtner
 * @version $Id: AWTPresenter.java 298 2014-09-30 12:52:15Z mcgaerty $
 *
 */
public interface AWTPresenter extends Presenter {

	/**
	 * Returns the {@code Component} this presenter is using to render
	 * its data. This method should never return {@code null} regardless
	 * of valid presentation data being set or not. This state is rather
	 * to be displayed by the visual {@code content} of the returned component.
	 */
	Component getPresentingComponent();

	public interface TableBasedPresenter extends AWTPresenter {
		// marker interface
	}

	public interface ListBasedPresenter extends AWTPresenter {
		// marker interface
	}

	public interface GraphBasedPresenter extends AWTPresenter {
		// marker interface
	}

	public interface TextBasedPresenter extends AWTPresenter {
		// marker interface
	}

	public interface ChartBasedPresenter extends AWTPresenter {
		// marker interface
	}
}
