/*
 *  ICARUS -  Interactive platform for Corpus Analysis and Research tools, University of Stuttgart
 *  Copyright (C) 2012-2013 Markus Gärtner and Gregor Thiele
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses.
 *
 * $Revision: 269 $
 * $Date: 2014-07-08 00:09:53 +0200 (Di, 08 Jul 2014) $
 * $URL: https://subversion.assembla.com/svn/icarusplatform/trunk/Icarus/core/de.ims.icarus/source/de/ims/icarus/util/SubstitutionSupport.java $
 *
 * $LastChangedDate: 2014-07-08 00:09:53 +0200 (Di, 08 Jul 2014) $
 * $LastChangedRevision: 269 $
 * $LastChangedBy: mcgaerty $
 */
package de.ims.icarus.util;

import gnu.trove.map.TObjectIntMap;
import gnu.trove.map.hash.TObjectIntHashMap;

import java.util.ArrayList;
import java.util.List;

/**
 *
 *
 * @author Markus Gärtner
 * @version $Id: SubstitutionSupport.java 269 2014-07-07 22:09:53Z mcgaerty $
 *
 */
public class SubstitutionSupport {

	protected final TObjectIntMap<String> substitutions;

	protected final List<String> resubstitutions;

	public SubstitutionSupport() {
		this(100);
	}

	public SubstitutionSupport(int size) {
		substitutions = new TObjectIntHashMap<>(size, 0.5f, -1);
		resubstitutions = new ArrayList<>(size);
	}

	public int substitute(String value) {
		int id = substitutions.get(value);
		if (id == -1) {
			id = resubstitutions.size();
			resubstitutions.add(value);
			substitutions.put(value, id);
		}

		return id;
	}

	public int getSubstitution(String value) {
		return substitutions.get(value);
	}

	public String resubstitute(int value) {
		return resubstitutions.get(value);
	}

	public void reset() {
		substitutions.clear();
		resubstitutions.clear();
	}

	public int size() {
		return resubstitutions.size();
	}

	public void set(String[] items) {
		substitutions.clear();
		resubstitutions.clear();

		for(int i=0; i<items.length; i++) {
			resubstitutions.add(items[i]);
			substitutions.put(items[i], i);
		}
	}
}
