/*
 *  ICARUS -  Interactive platform for Corpus Analysis and Research tools, University of Stuttgart
 *  Copyright (C) 2012-2013 Markus Gärtner and Gregor Thiele
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses.
 *
 * $Revision: 170 $
 * $Date: 2013-11-22 15:50:10 +0100 (Fr, 22 Nov 2013) $
 * $URL: https://subversion.assembla.com/svn/icarusplatform/trunk/Icarus/core/de.ims.icarus/source/de/ims/icarus/util/collections/BiDiMap.java $
 *
 * $LastChangedDate: 2013-11-22 15:50:10 +0100 (Fr, 22 Nov 2013) $ 
 * $LastChangedRevision: 170 $ 
 * $LastChangedBy: mcgaerty $
 */
package de.ims.icarus.util.collections;

import java.util.Collection;
import java.util.IdentityHashMap;
import java.util.Map;
import java.util.Set;

/**
 * @author Markus Gärtner
 * @version $Id: BiDiMap.java 170 2013-11-22 14:50:10Z mcgaerty $
 *
 */
public class BiDiMap<K extends Object, V extends Object> implements Map<K, V> {
	
	private Map<K, V> map;
	private Map<V, K> reverseMap;

	public BiDiMap() {
		map = new IdentityHashMap<>();
		reverseMap = new IdentityHashMap<>();
	}

	/**
	 * @see java.util.Map#size()
	 */
	@Override
	public synchronized int size() {
		return map.size();
	}

	/**
	 * @see java.util.Map#isEmpty()
	 */
	@Override
	public synchronized boolean isEmpty() {
		return map.isEmpty();
	}

	/**
	 * @see java.util.Map#containsKey(java.lang.Object)
	 */
	@Override
	public synchronized boolean containsKey(Object key) {
		return map.containsKey(key);
	}

	/**
	 * @see java.util.Map#containsValue(java.lang.Object)
	 */
	@Override
	public synchronized boolean containsValue(Object value) {
		return reverseMap.containsKey(value);
	}

	/**
	 * @see java.util.Map#get(java.lang.Object)
	 */
	@Override
	public synchronized V get(Object key) {
		return map.get(key);
	}
	
	public synchronized K getKey(Object value) {
		return reverseMap.get(value);
	}

	/**
	 * @see java.util.Map#put(java.lang.Object, java.lang.Object)
	 */
	@Override
	public synchronized V put(K key, V value) {
		V oldValue = map.put(key, value);
		reverseMap.put(value, key);
		
		return oldValue;
	}

	/**
	 * @see java.util.Map#remove(java.lang.Object)
	 */
	@Override
	public synchronized V remove(Object key) {
		V value = map.remove(key);
		if(value!=null) {
			reverseMap.remove(value);
		}
		return value;
	}
	
	public synchronized K removeValue(Object value) {
		K key = reverseMap.get(value);
		if(key!=null) {
			map.remove(key);
		}
		return key;
	}

	/**
	 * @see java.util.Map#putAll(java.util.Map)
	 */
	@Override
	public synchronized void putAll(Map<? extends K, ? extends V> m) {
		for(Entry<? extends K, ? extends V> entry : m.entrySet()) {
			map.put(entry.getKey(), entry.getValue());
			reverseMap.put(entry.getValue(), entry.getKey());
		}
	}

	/**
	 * @see java.util.Map#clear()
	 */
	@Override
	public synchronized void clear() {
		map.clear();
		reverseMap.clear();
	}

	/**
	 * @see java.util.Map#keySet()
	 */
	@Override
	public synchronized Set<K> keySet() {
		return map.keySet();
	}
	
	public synchronized Set<V> valueSet() {
		return reverseMap.keySet();
	}

	/**
	 * @see java.util.Map#values()
	 */
	@Override
	public synchronized Collection<V> values() {
		return reverseMap.keySet();
	}

	/**
	 * @see java.util.Map#entrySet()
	 */
	@Override
	public synchronized Set<Entry<K, V>> entrySet() {
		return map.entrySet();
	}
	
	public synchronized Set<Entry<V, K>> reverseEntrySet() {
		return reverseMap.entrySet();
	}
}
