/*
 *  ICARUS -  Interactive platform for Corpus Analysis and Research tools, University of Stuttgart
 *  Copyright (C) 2012-2013 Markus G�rtner and Gregor Thiele
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses.
 *
 * $Revision: 462 $
 * $Date: 2016-06-15 22:11:19 +0200 (Mi, 15 Jun 2016) $
 * $URL: https://subversion.assembla.com/svn/icarusplatform/trunk/Icarus/plugins/de.ims.icarus.plugins.errormining/source/de/ims/icarus/plugins/errormining/ngram_tools/NGramDataList.java $
 *
 * $LastChangedDate: 2016-06-15 22:11:19 +0200 (Mi, 15 Jun 2016) $
 * $LastChangedRevision: 462 $
 * $LastChangedBy: mcgaerty $
 */
package de.ims.icarus.plugins.errormining.ngram_tools;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.swing.event.ChangeListener;

import de.ims.icarus.language.AvailabilityObserver;
import de.ims.icarus.language.DataType;
import de.ims.icarus.language.Grammar;
import de.ims.icarus.language.SentenceData;
import de.ims.icarus.language.SentenceDataList;
import de.ims.icarus.language.dependency.DependencySentenceData;
import de.ims.icarus.language.dependency.DependencyUtils;
import de.ims.icarus.plugins.errormining.ItemInNuclei;
import de.ims.icarus.util.data.ContentType;

/**
 * @author Gregor Thiele
 * @version $Id: NGramDataList.java 462 2016-06-15 20:11:19Z mcgaerty $
 *
 */
public class NGramDataList implements SentenceDataList {


	protected Map<String,ArrayList<ItemInNuclei>> nGramMap;
	protected int index;
	protected List<Integer> sentences;
	protected List<SentenceData> corpus;
	protected Map<Integer, NewNGramSentenceData> nGramMapCache;
	//SentenceView
	protected List<CorpusType> corpusList;


	//For Debug with Cprus
	public NGramDataList(Map<String,ArrayList<ItemInNuclei>> nGramMap,
			List<SentenceData> corpus){
		if (nGramMap == null)
			throw new IllegalArgumentException("No Data"); //$NON-NLS-1$

		this.corpus = corpus;
		setNGramMap(nGramMap);

	}

	// w/o Corpus for NGramSearch Only
	public NGramDataList(Map<String,ArrayList<ItemInNuclei>> nGramMap){
		if (nGramMap == null)
			throw new IllegalArgumentException("No Data"); //$NON-NLS-1$

		setNGramMap(nGramMap);
	}


	/**
	 * @return the nGramMap
	 */
	public Map<String, ArrayList<ItemInNuclei>> getnGramMap() {
		return nGramMap;
	}



	/**
	 * @return the corpus
	 */
	public List<SentenceData> getCorpus() {
		return corpus;
	}

	/**
	 * @param corpus the corpus to set
	 */
	public void setCorpus(List<SentenceData> corpus) {
		this.corpus = corpus;
	}




	public List<CorpusType> getCorpusList(){
		return corpusList;
	}

	/**
	 * @param nGramMap
	 */
	void setNGramMap(Map<String, ArrayList<ItemInNuclei>> nGramMap) {
		if (this.nGramMap != null) {
			return;
		}
		nGramMapCache = new HashMap<Integer, NewNGramSentenceData>();

		//TODO filter results
		this.nGramMap = nGramMap;

		filterNGramMap(nGramMap);

	}


	private Map<String, ArrayList<ItemInNuclei>> filterNGramMap(Map<String, ArrayList<ItemInNuclei>> nGramMap){

		List<String> tmpKey = new ArrayList<String>(nGramMap.keySet());
		Collections.reverse(tmpKey);

		corpusList = new ArrayList<CorpusType>();
		sentences = new ArrayList<Integer>();

		Map<String, ArrayList<ItemInNuclei>> filterMap = new LinkedHashMap<String,ArrayList<ItemInNuclei>>();

//
//		for(int j = 0; j < tmpKey.size(); j++){
//			System.out.println();
//			System.out.println("key" + tmpKey.get(j));
//		}


		for(int i = 0; i < tmpKey.size(); i++){
			String key = tmpKey.get(i);
			String[] keysplit = key.split(" "); //$NON-NLS-1$

			//System.out.println(key + keysplit.length);

			ArrayList<ItemInNuclei> value = nGramMap.get(key);
			for (int j = 0; j < value.size();j++){
				ItemInNuclei iin = value.get(j);
//				System.out.println("PoSTag: "+ iin.getPosTag() +
//								  " PoSCount: " + iin.getCount());

					for (int k = 0; k < iin.getSentenceInfoSize(); k++){
						int sentenceNR = iin.getSentenceInfoAt(k).getSentenceNr()-1;
//						System.out.println(key + " " + sentenceNR);
						if(sentences.contains(sentenceNR)){
							//donothing
						} else {
							//wenn nicht nur nuclei in liste aufnehmen
							//TODO maybe workaround when only nuclei displayed
							if(keysplit.length > 1){
								filterMap.put(key, value);
								corpusList.add(new CorpusType(sentenceNR, key));
								sentences.add(sentenceNR);
							}

						}
					}
			}
		}


//		for (int i = 0; i < sentences.size(); i++){
//			System.out.println("Satz: " + sentences.get(i));
//		}

		// for(Iterator<String> i = filterMap.keySet().iterator(); i.hasNext();){
		//		String key = i.next();
		//		ArrayList<ItemInNuclei> arrItem = filterMap.get(key);
		//
		//		System.out.println("\n### Wordform: " + key + " ###");
		//		for (int j = 0; j < arrItem.size();j++){
		//			ItemInNuclei iin = arrItem.get(j);
		//			System.out.println("PoSTag: "+ iin.getPosTag() +
		//							  "  PoSCount: " + iin.getCount());
		//
		//			for (int k = 0; k < iin.getSentenceInfoSize(); k++){
		//				System.out.print("SentenceNr: " + iin.getSentenceInfoAt(k).getSentenceNr());
		//				System.out.print(" | NucleiCount: " + iin.getSentenceInfoAt(k).getNucleiIndexListSize());
		//				System.out.print(" NucleiIndex: " + iin.getSentenceInfoAt(k).getNucleiIndex());
		//				//System.out.print(" NucleiPos: "); printNuclei(iin.getSentenceInfoAt(k));
		//				System.out.print(" Begin: " + iin.getSentenceInfoAt(k).getSentenceBegin());
		//				System.out.println(" End: " + iin.getSentenceInfoAt(k).getSentenceEnd());
		//			}
		//		}
		//	}

		//remaining keys in filtered map
//		List<String> tmpK = new ArrayList<String>(filterMap.keySet());
//		Collections.reverse(tmpKey);
//		for(int k = 0; k < tmpK.size(); k++){
//			System.out.println("key " + tmpK.get(k));
//		}

		return filterMap;
	}

	//no more needed?
//	private Map<String, ArrayList<ItemInNuclei>> getNGramMap(){
//		return nGramMap;
//	}
//
//
//	private List<Integer> getSentences(){
//		return sentences;
//	}


	private SentenceData getNGramDataFromIndex(int index) {
		if (!nGramMapCache.containsKey(index)) {
			NewNGramSentenceData ngramData = new NewNGramSentenceData(index);
			nGramMapCache.put(index, ngramData);
		}
		return nGramMapCache.get(index);
	}


	/**
	 * @see net.ikarus_systems.icarus.util.data.DataList#size()
	 */
	@Override
	public int size() {
		//return getNGramMap().size();
		return sentences.size();
	}

	public int getSentenceAtIndex(int i){
		return sentences.get(i);
	}

	/**
	 * @see net.ikarus_systems.icarus.util.data.DataList#get(int)
	 */
	@Override
	public SentenceData get(int index) {
		return get(index, DataType.SYSTEM);
	}

	/**
	 * @see net.ikarus_systems.icarus.util.data.DataList#getContentType()
	 */
	@Override
	public ContentType getContentType() {
		return DependencyUtils.getDependencyContentType();
	}

	/**
	 * @see net.ikarus_systems.icarus.util.data.DataList#addChangeListener(javax.swing.event.ChangeListener)
	 */
	@Override
	public void addChangeListener(ChangeListener listener) {
		// TODO Auto-generated method stub
	}

	/**
	 * @see net.ikarus_systems.icarus.util.data.DataList#removeChangeListener(javax.swing.event.ChangeListener)
	 */
	@Override
	public void removeChangeListener(ChangeListener listener) {
		// TODO Auto-generated method stub
	}

	/**
	 * @see net.ikarus_systems.icarus.language.SentenceDataList#supportsType(net.ikarus_systems.icarus.language.DataType)
	 */
	@Override
	public boolean supportsType(DataType type) {
		return type == DataType.SYSTEM;
	}

	/**
	 * @see net.ikarus_systems.icarus.language.SentenceDataList#get(int, net.ikarus_systems.icarus.language.DataType)
	 */
	@Override
	public SentenceData get(int index, DataType type) {
		if (type != DataType.SYSTEM) {
			return null;
		}
		return sentences == null ? null : getNGramDataFromIndex(index);
	}

	/**
	 * @see net.ikarus_systems.icarus.language.SentenceDataList#get(int, net.ikarus_systems.icarus.language.DataType, net.ikarus_systems.icarus.language.AvailabilityObserver)
	 */
	@Override
	public SentenceData get(int index, DataType type,
			AvailabilityObserver observer) {
		return get(index, type);
	}




//	//data class
//	private class NewNGramSentenceData implements AnnotatedSentenceData {
//
//		private static final long serialVersionUID = 3303973536847711267L;
//
//		private List<DependencyDataEntry> items = new ArrayList<>();
//
//		protected Annotation annotation = null; // TODO change to default value?
//
//		DependencySentenceData dd;
//
//		public NewNGramSentenceData(int index){
//			dd = (DependencySentenceData) corpus.get(index);
//
//
//		}
//
//
//		//TODO
//		@Override
//		public NewNGramSentenceData clone() {
//			return this;
//		}
//
//
//
//		/**
//		 * @see net.ikarus_systems.icarus.language.SentenceData#getForm(int)
//		 */
//		@Override
//		public String getForm(int index) {
//			//return items.get(index).getForm();
//			return dd.getForm(index);
//		}
//
//		/**
//		 * @see net.ikarus_systems.icarus.language.SentenceData#isEmpty()
//		 */
//		@Override
//		public boolean isEmpty() {
//			//return items.isEmpty();
//			return dd == null || dd.isEmpty();
//		}
//
//		/**
//		 * @see net.ikarus_systems.icarus.language.SentenceData#length()
//		 */
//		@Override
//		public int length() {
//			//return items.size();
//			return dd.length();
//		}
//
//		/**
//		 * @see net.ikarus_systems.icarus.language.SentenceData#getSourceGrammar()
//		 */
//		@Override
//		public Grammar getSourceGrammar() {
//			return DependencyUtils.getDependencyGrammar();
//		}
//
//		/**
//		 * @see net.ikarus_systems.icarus.language.annotation.AnnotatedSentenceData#getAnnotation()
//		 */
//		@Override
//		public Annotation getAnnotation() {
//			return annotation;
//		}
//
//
//		/**
//		 * @see net.ikarus_systems.icarus.ui.helper.TextItem#getText()
//		 */
//		@Override
//		public String getText() {
//			return dd.getText();
//		}
//
//	}



	/**
	 *
	 * @author Gregor Thiele
	 * @version $Id: NGramDataList.java 462 2016-06-15 20:11:19Z mcgaerty $
	 *
	 */


	private class NewNGramSentenceData implements DependencySentenceData {


		private static final long serialVersionUID = 4937843525979650838L;

		DependencySentenceData dd;

		public NewNGramSentenceData(int index) {

			dd = (DependencySentenceData) corpus.get(index);

		}

		@Override
		public NewNGramSentenceData clone() {
			return this;
		}
		/**
		 * @see de.ims.icarus.language.SentenceData#isEmpty()
		 */
		@Override
		public boolean isEmpty() {
			return dd == null || dd.isEmpty();
		}

		/**
		 * @see de.ims.icarus.language.SentenceData#length()
		 */
		@Override
		public int length() {
			return dd.length();
		}

		/**
		 * @see de.ims.icarus.language.SentenceData#getSourceGrammar()
		 */
		@Override
		public Grammar getSourceGrammar() {
			return dd.getSourceGrammar();
		}

		/**
		 * @see de.ims.icarus.ui.text.TextItem#getText()
		 */
		@Override
		public String getText() {
			return dd.getText();
		}

		/**
		 * @see de.ims.icarus.language.dependency.DependencySentenceData#getForm(int)
		 */
		@Override
		public String getForm(int index) {
			return dd.getForm(index);
		}

		/**
		 * @see de.ims.icarus.language.dependency.DependencySentenceData#getPos(int)
		 */
		@Override
		public String getPos(int index) {
			return dd.getPos(index);
		}

		/**
		 * @see de.ims.icarus.language.dependency.DependencySentenceData#getRelation(int)
		 */
		@Override
		public String getRelation(int index) {
			return dd.getRelation(index);
		}

		/**
		 * @see de.ims.icarus.language.dependency.DependencySentenceData#getLemma(int)
		 */
		@Override
		public String getLemma(int index) {
			return dd.getLemma(index);
		}

		/**
		 * @see de.ims.icarus.language.dependency.DependencySentenceData#getFeatures(int)
		 */
		@Override
		public String getFeatures(int index) {
			return dd.getFeatures(index);
		}

		/**
		 * @see de.ims.icarus.language.dependency.DependencySentenceData#getHead(int)
		 */
		@Override
		public int getHead(int index) {
			return dd.getHead(index);
		}

		/**
		 * @see de.ims.icarus.language.dependency.DependencySentenceData#isFlagSet(int, long)
		 */
		@Override
		public boolean isFlagSet(int index, long flag) {
			return dd.isFlagSet(index, flag);
		}

		/**
		 * @see de.ims.icarus.language.dependency.DependencySentenceData#getFlags(int)
		 */
		@Override
		public long getFlags(int index) {
			return dd.getFlags(index);
		}

		/**
		 * @see de.ims.icarus.language.SentenceData#getIndex()
		 */
		@Override
		public int getIndex() {
			return dd.getIndex();
		}

		/**
		 * @see de.ims.icarus.language.SentenceData#getProperty(int, java.lang.String)
		 */
		@Override
		public Object getProperty(int index, String key) {
			return null;
		}

		/**
		 * @see de.ims.icarus.language.SentenceData#getProperty(java.lang.String)
		 */
		@Override
		public Object getProperty(String key) {
			return null;
		}

	}


}
