/*
 *  ICARUS -  Interactive platform for Corpus Analysis and Research tools, University of Stuttgart
 *  Copyright (C) 2012-2013 Markus Gärtner and Gregor Thiele
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses.
 *
 * $Revision: 389 $
 * $Date: 2015-04-23 12:19:15 +0200 (Do, 23 Apr 2015) $
 * $URL: https://subversion.assembla.com/svn/icarusplatform/trunk/Icarus/plugins/de.ims.icarus.plugins.search_tools/source/de/ims/icarus/plugins/search_tools/view/graph/ConstraintEdgeData.java $
 *
 * $LastChangedDate: 2015-04-23 12:19:15 +0200 (Do, 23 Apr 2015) $
 * $LastChangedRevision: 389 $
 * $LastChangedBy: mcgaerty $
 */
package de.ims.icarus.plugins.search_tools.view.graph;

import java.util.ArrayList;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlRootElement;

import de.ims.icarus.search_tools.EdgeType;
import de.ims.icarus.search_tools.SearchEdge;
import de.ims.icarus.search_tools.util.SearchUtils;
import de.ims.icarus.util.collections.CollectionUtils;


/**
 * @author Markus Gärtner
 * @version $Id: ConstraintEdgeData.java 389 2015-04-23 10:19:15Z mcgaerty $
 *
 */
@XmlRootElement(name="edgeConstraints")
@XmlAccessorType(XmlAccessType.FIELD)
public class ConstraintEdgeData extends ConstraintCellData<ConstraintEdgeData> implements Cloneable {

	private static final long serialVersionUID = 3694619821568182574L;

	@XmlAttribute
	private EdgeType edgeType = EdgeType.DOMINANCE;

	public ConstraintEdgeData() {
		// no-op
	}

	public ConstraintEdgeData(ConstraintEdgeData source) {
		copyFrom(source);
	}

	public ConstraintEdgeData(SearchEdge source) {
		copyFrom(source);
	}

	public ConstraintEdgeData(int size) {
		constraints = new ArrayList<>(size);
	}

	/**
	 *
	 * @see de.ims.icarus.plugins.search_tools.view.graph.ConstraintCellData#copyFrom(de.ims.icarus.plugins.search_tools.view.graph.ConstraintCellData)
	 */
	@Override
	public void copyFrom(ConstraintEdgeData source) {
		negated = source.negated;
		edgeType = source.edgeType;
		id = source.id;
		constraints = SearchUtils.cloneConstraints(source.constraints);
	}

	public void copyFrom(SearchEdge source) {
		negated = source.isNegated();
		edgeType = source.getEdgeType();
		id = source.getId();
		constraints = CollectionUtils.asList(SearchUtils.cloneConstraints(source.getConstraints()));
	}

	/**
	 *
	 * @see de.ims.icarus.plugins.search_tools.view.graph.ConstraintCellData#clone()
	 */
	@Override
	public ConstraintEdgeData clone() {
		return new ConstraintEdgeData(this);
	}

	public EdgeType getEdgeType() {
		return edgeType;
	}

	public void setEdgeType(EdgeType edgeType) {
		this.edgeType = edgeType;
	}
}
