/*
 *  ICARUS -  Interactive platform for Corpus Analysis and Research tools, University of Stuttgart
 *  Copyright (C) 2012-2013 Markus Gärtner and Gregor Thiele
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses.
 *
 * $Revision: 159 $
 * $Date: 2013-11-04 11:20:05 +0100 (Mo, 04 Nov 2013) $
 * $URL: https://subversion.assembla.com/svn/icarusplatform/trunk/Icarus/plugins/de.ims.icarus.plugins.search_tools/source/de/ims/icarus/search_tools/annotation/AnnotationBuffer.java $
 *
 * $LastChangedDate: 2013-11-04 11:20:05 +0100 (Mo, 04 Nov 2013) $ 
 * $LastChangedRevision: 159 $ 
 * $LastChangedBy: mcgaerty $
 */
package de.ims.icarus.search_tools.annotation;

import de.ims.icarus.search_tools.result.ResultEntry;
import de.ims.icarus.search_tools.result.SearchResult;
import de.ims.icarus.util.annotation.AnnotatedData;
import de.ims.icarus.util.annotation.AnnotationContainer;
import de.ims.icarus.util.cache.LRUCache;
import de.ims.icarus.util.data.ContentType;

/**
 * @author Markus Gärtner
 * @version $Id: AnnotationBuffer.java 159 2013-11-04 10:20:05Z mcgaerty $
 *
 */
public class AnnotationBuffer implements AnnotationContainer {
	
	private final SearchResult searchResult;
	private final ResultAnnotator annotator;
	private final LRUCache<ResultEntry, AnnotatedData> cache;

	public AnnotationBuffer(SearchResult searchResult, ResultAnnotator annotator, int cacheSize) {
		if(searchResult==null)
			throw new NullPointerException("Invalid search result"); //$NON-NLS-1$
		if(annotator==null)
			throw new NullPointerException("Invalid result annotator"); //$NON-NLS-1$
		
		this.searchResult = searchResult;
		this.annotator = annotator;
		cache = new LRUCache<>(cacheSize);
	}

	public void setCacheSize(int cacheSize) {
		cache.setMaxSize(cacheSize);
	}
	
	public AnnotatedData getAnnotatedData(ResultEntry entry) {
		AnnotatedData annotatedData = cache.get(entry);
		
		if(annotatedData==null) {
			Object data = searchResult.getPlainEntry(entry);
			if(data!=null) {
				annotatedData = annotator.annotate(searchResult, data, entry);
				cache.put(entry, annotatedData);
			}
		}
		
		return annotatedData;
	}
	
	public void clear() {
		cache.clear();
	}

	/**
	 * @see de.ims.icarus.util.annotation.AnnotationContainer#getAnnotationType()
	 */
	@Override
	public ContentType getAnnotationType() {
		return annotator.getAnnotationType();
	}
}
