package ims;

import java.io.File;
import java.io.PrintStream;
import java.util.Date;

public class AnnaPipelineOptions {

	public File lemmaModel;
	public File tagModel;
	public File morphModel;
	public File parseModel;
	
	public File in;
	public File out;
	
	public File eval;
	
//	public int cores=Math.max(Runtime.getRuntime().availableProcessors()-1,1);
	public int cores=(int) Math.max(1, Math.floor(Runtime.getRuntime().availableProcessors()/2.0));
	
	public int newLineInterval=500;
	
	public boolean doUppercaseLemmas=false;
	
	
	public final Date creationDate=new Date();
	
	
	public AnnaPipelineOptions(String[] args){
		parseCmdLineArgs(args);
		sanityCheck();
	}

	private void sanityCheck() {
		if(in==null)
			die("need to specify inputfile -in");
		if(!in.exists() || !in.canRead())
			die("input file "+in+" does not exist, or cannot be read");
		if(out==null)
			die("need to specifc outputfile -out");
//		if(!out.canWrite())
//			die("cannot write to output "+out);
		if(lemmaModel!=null && (!lemmaModel.exists() || !lemmaModel.canRead()))
			die("lemma model "+lemmaModel+" does not exist, or cannot be read");
		if(tagModel!=null && (!tagModel.exists() || !tagModel.canRead()))
			die("tagger model "+tagModel+" does not exist, or cannot be read");
		if(morphModel!=null && (!morphModel.exists() || !morphModel.canRead()))
			die("morph tagger model "+morphModel+" does not exist, or cannot be read");
		if(parseModel!=null && (!parseModel.exists() || !parseModel.canRead()))
			die("parser model "+parseModel+" does not exist, or cannot be read");
	}

	private void parseCmdLineArgs(String[] args) {
		int ai=0;
		while(ai<args.length){
			if(args[ai].equals("-in")){
				in=new File(args[++ai]);
			} else if(args[ai].equals("-out")){
				out=new File(args[++ai]);
			} else if(args[ai].equals("-lemma") || args[ai].equals("-l")){
				lemmaModel=new File(args[++ai]);
			} else if(args[ai].equals("-tag") || args[ai].equals("-t")){
				tagModel=new File(args[++ai]);
			} else if(args[ai].equals("-morph") || args[ai].equals("-m")){
				morphModel=new File(args[++ai]);
			} else if(args[ai].equals("-parse") || args[ai].equals("-p")){
				parseModel=new File(args[++ai]);
			} else if(args[ai].equals("-cores")){
				cores=Integer.parseInt(args[++ai]);
			} else if(args[ai].equals("-eval")){
				eval=new File(args[++ai]);
			} else if(args[ai].equals("-upperCaseLemmas")){
				doUppercaseLemmas=true;
			} else if(args[ai].equals("-newLineInterval")){
				newLineInterval=Integer.parseInt(args[++ai]);
				if(newLineInterval<1)
					die("cannot have new line interval < 1");
			} else {
				die("failed to parse cmd line argument '"+args[ai]+"'");
			}
			ai++;
		}
	}

	private static void die(String string) {
		System.err.println(string);
		System.err.println();
		usage(System.err);
		System.exit(1);
	}
	
	private static void usage(PrintStream out){
		out.println("Usage: ");
		out.println("java -cp ... ims.AnnaPipeline <args>, where <args> is:");
		out.println("-in <input-file>                MANDATORY");
		out.println("-out <output-file>              MANDATORY");
		out.println("-l (-lemma) <lemma-model>       OPTIONAL");
		out.println("-t (-tag)   <tagger-model>      OPTIONAL");
		out.println("-m (-morph) <morph-tag-model>   OPTIONAL");
		out.println("-p (-parse) <parser-model>      OPTIONAL");
		out.println("-cores <cores>                  OPTIONAL (defaults to half of available)");
		out.println("-upperCaseLemmas                OPTIONAL (default to false)");
		out.println("-newLineInterval                OPTIONAL (default 500)");
		out.println("-eval <gold-file>               OPTIONAL");
		out.println();
		out.println("if the input or output files have a suffix .gz, then reading/writing will automatically be GZipped");
		out.println();
	}
}
