package ims.gw;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;
import java.util.zip.GZIPOutputStream;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import edu.stanford.nlp.ling.HasWord;
import edu.stanford.nlp.process.DocumentPreprocessor;
import edu.stanford.nlp.process.PTBTokenizer;
import edu.stanford.nlp.process.PTBTokenizer.PTBTokenizerFactory;

public abstract class AbstractDefaultHandler extends DefaultHandler {
	
	static enum State { Keep, Toss }
	static enum InP { Yes, No }

	int docCount=0;
	int keepDocCount=0;
	
	int senCount=0;
	int keepSenCount=0;
	
	int keepTokenCount=0;
	
	State state=State.Toss;
	InP inp=InP.No;
	StringBuffer sb=null;

	Filter filter;
	TokenMassager tokenMassager;
	
	PrintStream out;
	
	AbstractDefaultHandler(Options options) throws IOException{
		this.filter=new Filter(options);
		this.tokenMassager=new TokenMassager(options.doNormalize);
		//Setup the printstream
		OutputStream os;
		if(options.output==null){
			os=System.out;
		} else {
			os=new FileOutputStream(options.output);
		}
		if(options.gzippedOut)
			os=new GZIPOutputStream(os);
		this.out=new PrintStream(os,false,options.outputEnc);
	}
	
	abstract void startDoc(String docType);
	abstract void handleParagraph(List<List<String>> asStrings);
	protected abstract void subclassFinish();
	
	public void startElement(String uri, String localName,String qName, Attributes attributes) throws SAXException{
		if(qName.equals("DOC")){
			docCount++;
			String type=attributes.getValue("type");
			startDoc(type);
			if(type.equals("story")){
				state=State.Keep;
				keepDocCount++;
			}
		} else if(qName.equals("P")){
			sb=new StringBuffer();
			inp=InP.Yes;
		}
	}
	
	public void characters(char[] ch, int start, int length){
		if(state==State.Keep && inp==InP.Yes)
			sb.append(ch, start, length);
	}

	public void endElement(String uri, String localName, String qName){
		if(qName.equals("DOC")){
			state=State.Toss;
		} else if(qName.equals("P")){
			//End of P, do whatever we wanna do to the paragraph
			String paragraph=sb.toString();
			splitAndHandle(paragraph);
			inp=InP.No;
			sb=null;
		}
	}

	private void splitAndHandle(String paragraph) {
		if(filter.tossParagraph(paragraph))
			return;
		StringReader reader=new StringReader(paragraph);
		DocumentPreprocessor dp = new DocumentPreprocessor(reader);
		dp.setTokenizerFactory(ptbTokenizerFactory);
		List<List<HasWord>> sentences=new ArrayList<List<HasWord>>();
		for(List<HasWord> sent:dp){
			senCount++;
			if(filter.tossSentence(sent))
				continue;
			keepSenCount++;
			sentences.add(sent);
		}
		List<List<String>> asStrings=tokenMassager.hasWordToStringSentences(sentences);
		for(List<String> l:asStrings)
			keepTokenCount+=l.size();
		handleParagraph(asStrings);
	}

	public void finish() {
		out.flush();
		out.close();
		subclassFinish();
		System.err.println("---------------");
		System.err.println("Read docs: "+docCount);
		System.err.println("Kept docs: "+keepDocCount);
		System.err.println("Read sents: "+senCount);
		System.err.println("Kept sents: "+keepSenCount);
		System.err.println("Kept tokens: "+keepTokenCount);
	}
	
	static final PTBTokenizerFactory<? extends HasWord> ptbTokenizerFactory;
	static {
		StringBuilder sb=new StringBuilder();
		sb.append("normalizeParentheses=false,normalizeOtherBrackets=false,untokenizable=allKeep,escapeForwardSlashAsterisk=false");
		String options=sb.toString();
		ptbTokenizerFactory=PTBTokenizer.PTBTokenizerFactory.newWordTokenizerFactory(options);
	}
}