package ims.gw;

import ims.gw.Options.OutputFormat;

import java.io.IOException;
import java.util.List;

public class CleanAndTokenize extends AbstractDefaultHandler {

	final OutputFormat outFormat;
	final boolean numberSentences;

	
	CleanAndTokenize(Options options) throws IOException {
		super(options);
		outFormat=options.outputFormat;
		numberSentences=options.includeSentenceNumberInCoNLLOutput;
	}	

	void handleParagraph(List<List<String>> asStrings){
		switch(outFormat){
		case OneSentencePerLine: 	outOneSentencePerLine(asStrings); return;
		case OneWordPerLine:		outOneWordPerLine(asStrings); return;
		case CoNLL06:				outCoNLL06(asStrings); return;
		case CoNLL09:				outCoNLL09(asStrings); return;
		default: throw new RuntimeException("not implemented");
		}
	}


	private void outCoNLL09(List<List<String>> dp) {
		int s_ind=senCount-dp.size();
		for (List<String> sentence : dp) {
			++s_ind;
			int index=1;
			for(String hw:sentence){
				StringBuilder sb=getCoNLLSB(index++,s_ind);
				sb.append('\t').append(hw);
				appendUnderscores(sb,12);
				out.println(sb.toString());
			}
			out.println();
		}
	}

	private void outCoNLL06(List<List<String>> dp) {
		int s_ind=senCount-dp.size();
		for (List<String> sentence : dp) {
			++s_ind;
			int index=1;
			for(String hw:sentence){
				StringBuilder sb=getCoNLLSB(index++,s_ind);
				sb.append('\t').append(hw);
				appendUnderscores(sb,8);
				out.println(sb.toString());
			}
			out.println();
		}
	}

	private StringBuilder getCoNLLSB(int index,int s_ind) {
		if(numberSentences)
			return new StringBuilder(Integer.toString(s_ind)).append("_").append(Integer.toString(index));
		else
			return new StringBuilder(Integer.toString(index));
	}

	private static void appendUnderscores(StringBuilder sb, int count) {
		for(int i=0;i<count;++i)
			sb.append('\t').append('_');
	}


	private void outOneWordPerLine(List<List<String>> dp) {
		for (List<String> sentence : dp) {
			for(String hw:sentence)
				out.println(hw);
			out.println();
		}
	}


	private void outOneSentencePerLine(List<List<String>> dp) {
		for (List<String> sentence : dp) {
			StringBuilder sb=new StringBuilder();
			for(String hw:sentence)
				sb.append(hw).append(' ');
			out.println(sb.toString().trim());
		}
	}


	@Override
	void startDoc(String docType) {
		//Do nothing
	}


	@Override
	protected void subclassFinish() {
		//Do nothing
	}
}
