package ims.gw;

import java.nio.charset.Charset;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import edu.stanford.nlp.ling.HasWord;

public class Filter {

	private final boolean tossBeginLowercase;
	private final boolean tossIfNotCorrectEnd;
	private final boolean tossUnbalancedQuotes;
	private final boolean tossNonASCII;
	private final boolean tossUnbalancedBrackets;
//	private final boolean tossBackslashSlash;
	private final int minSenLength;
	private final int maxSenLength;
	
	
	public Filter(Options options){
		this.tossBeginLowercase=options.tossBeginLowercase;
		this.tossIfNotCorrectEnd=options.tossWrongFinalSentenceTokens;
		this.tossUnbalancedQuotes=options.tossUnbalancedQuotes;
		this.tossNonASCII=options.tossNonASCII;
//		this.tossBackslashSlash=options.tossBackslashSlash;
		this.tossUnbalancedBrackets=options.tossUnbalancedBrackets;
		this.minSenLength=options.minSenLen;
		this.maxSenLength=options.maxSenLen;
	}
	
	public boolean tossParagraph(String paragraph){
		if(tossNonASCII && hasNonASCII(paragraph))
			return true;
		return false;
	}
	
	public boolean tossSentence(List<HasWord> sentence){
		if(tossBeginLowercase && doTossBeginLowercase(sentence))
			return true;
		if(tossIfNotCorrectEnd && doTossNotCorrectEnd(sentence))
			return true;
		if(tossUnbalancedQuotes && hasUnbalancedQuotes(sentence))
			return true;
		if(tossUnbalancedBrackets && hasUnbalancedBrackets(sentence))
			return true;
//		if(tossBackslashSlash && hasBackslashSlash(sentence))
//			return true;
		if(sentence.size()<minSenLength || sentence.size()>maxSenLength)
			return true;
		return false;
	}



	private boolean hasBackslashSlash(List<HasWord> sentence) {
		for(HasWord hw:sentence)
			if(hw.toString().contains("\\/"))
					return true;
		return false;
	}

	static final Charset UTF8_CHARSET=Charset.forName("UTF8");
	private boolean hasNonASCII(String s) {
		byte[] bytes=s.getBytes(UTF8_CHARSET);
		for(byte b:bytes)
			if(b<0x20 && b!=10 && b!=8 || b>0x80)
				return true;
		return false;
	}
	
	private boolean hasUnbalancedBrackets(List<HasWord> sentence) {
		int round=0;
		int square=0;
		int curly=0;
		for(int i=0;i<sentence.size();++i){
			String form=sentence.get(i).toString();
			if(form.equals("("))
				round++;
			if(form.equals(")"))
				round--;
			if(form.equals("["))
				square++;
			if(form.equals("]"))
				square--;
			if(form.equals("{"))
				curly++;
			if(form.equals("}"))
				curly--;
		}
		return round!=0 || curly!=0 || square!=0;
	}

	private boolean hasUnbalancedQuotes(List<HasWord> sentence) {
		int qCount=0;
		for(int i=0;i<sentence.size();++i){
			String form=sentence.get(i).toString();
			if(form.equals("``")) //Open
				qCount++;
			if(form.equals("''"))
				qCount--;
			if(qCount<0)
				return false;
		}
		return qCount!=0;
	}

	private static final Set<String> ALLOWED_FINAL_WORDS=new HashSet<String>();
	private static final String[] ALLOWED_FINAL_WORDS_ARR={".","!","''"};
	static {
		for(String s:ALLOWED_FINAL_WORDS_ARR)
			ALLOWED_FINAL_WORDS.add(s);
	}
	private boolean doTossNotCorrectEnd(List<HasWord> sentence) {
		String last=sentence.get(sentence.size()-1).toString();
		if(ALLOWED_FINAL_WORDS.contains(last))
			return false;
		else
			return true;
	}

	private boolean doTossBeginLowercase(List<HasWord> sentence) {
		String s=sentence.get(0).toString();
		return Character.isLowerCase(s.charAt(0));
	}
	
}
