package ims.gw;

import java.io.File;
import java.io.OutputStream;

import org.kohsuke.args4j.CmdLineException;
import org.kohsuke.args4j.CmdLineParser;
import org.kohsuke.args4j.Option;

public class Options {

	public static enum OutputFormat { OneSentencePerLine, OneWordPerLine, CoNLL06, CoNLL09 }
	public static enum Mode { Stats, Clean }
	
	@Option(name="-in",aliases="--input",usage="input file (- means read from STDIN)\nInput file may be gzipped (then filename should end with .gz, or the --gzipInput option should be passed)")
	void setInput(String input){
		if(input.equals("-"))
			return;
		File inFile=new File(input);
		if(!inFile.exists())
			throw new RuntimeException("Input file "+input+" does not exist.");
		if(input.toString().endsWith(".gz"))
			gzippedIn=true;
		this.input=inFile;
	}
	@Option(name="-out",aliases="--output",usage="output file (- means STDOUT)\nOutput may be gzipped (If file name ends with .gz, or the --gzipOutput option is set")
	void setOutput(String output){
		if(output.equals("-"))
			return;
		if(output.toString().endsWith(".gz"))
			gzippedOut=true;
		this.output=new File(output);
	}
	
	public File input;
	public File output;
	@Option(name="-gzin",aliases="--gzipInput",usage="input is gzipped")
	public boolean gzippedIn;
	@Option(name="-gzout",aliases="--gzipOutput",usage="output is gzipped")
	public boolean gzippedOut;
	@Option(name="-inEnc",aliases="--inputEncoding",usage="File encoding on input")
	public String inputEnc="UTF8";
	@Option(name="-outEnc",aliases="--outputEncoding",usage="File encoding on output")
	public String outputEnc="UTF8";
	
	@Option(name="-format",aliases="--outputFormat",usage="The format for the output.")
	public OutputFormat outputFormat=OutputFormat.CoNLL09;
	@Option(name="-numberSentences",usage="Includes sentence numbers in the CoNLL outputs")
	public boolean includeSentenceNumberInCoNLLOutput=true;
	@Option(name="-mode",usage="Mode -- Clean and print, or just count and print Stats")
	public Mode mode=Mode.Stats;
	@Option(name="-maxSenLen",usage="Maximum length of sentences to keep (default 60)")
	public int maxSenLen=60;
	@Option(name="-minSenLen",usage="Minimum length of sentences to keep (default 6)")
	public int minSenLen=6;
	@Option(name="-tossBeginLowercase",usage="Throw out sentences that begin with a lowercase character")
	public boolean tossBeginLowercase=true;
	@Option(name="-tossWrongFinalSentenceTokens",usage="Throw out sentences that don't end with predefined final words")
	public boolean tossWrongFinalSentenceTokens=true;
	@Option(name="-tossUnbalancedQuotes",usage="Throw out sentences that have unbalanced number of quotes")
	public boolean tossUnbalancedQuotes=true;
	@Option(name="-tossUnbalancedBrackes",usage="throw out sentences that have unbalanced number of brackets")
	public boolean tossUnbalancedBrackets=true;
	@Option(name="-tossNonASCII",usage="Throw out sentences with non-ASCII chars")
	public boolean tossNonASCII=true;
	@Option(name="-tossBackslashSlash",usage="Throw out sentences with \\/")
	public boolean tossBackslashSlash=true;
	@Option(name="-normalize",usage="whether to apply surface form normalizations (e.g. numbers and URLs)")
	public boolean doNormalize=true;
	@Option(name="-help",aliases="--help",usage="Display help")
	public boolean help=false;
	
	public Options(String[] args){
		CmdLineParser clp=new CmdLineParser(this);
		try {
			clp.parseArgument(args);
		} catch (CmdLineException e) {
			e.printStackTrace();
			System.err.println("Failed to parse cmd line, aborting.");
			System.err.println();
			System.err.println("Valid options are:");
			printUsage(System.err);
			System.exit(1);
		}
		if(help){
			printUsage(System.out);
			System.exit(1);
		}
	}
	
	
	private Options(){}
	public static void printUsage(OutputStream out){
		CmdLineParser clp=new CmdLineParser(new Options());
		clp.printUsage(out);
	}
	
	public static void main(String[] args){
		printUsage(System.out);
	}
}
