package ims.gw;

import ims.normalization.TokenNormalizer;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.regex.Pattern;

import edu.stanford.nlp.ling.HasWord;

public class TokenMassager {

	public final boolean doNormalize;
	
	public TokenMassager(boolean doNormalize){
		this.doNormalize=doNormalize;
	}
	
	public List<String> hasWordToString(List<HasWord> sent){
		List<String> r=new ArrayList<String>();
		for(HasWord hw:sent){
			String hwString=hw.toString();
			if(hwString.contains("-") && hwString.length()>1){
				String[] split=handleHyphens(hwString);
				r.add(split[0]);
				for(int i=1;i<split.length;++i){
					r.add("-");
					r.add(split[i]);
				}
			} else {
				r.add(hw.toString());
			}
		}
		return r;
	}
    static final Pattern POTENTIAL_SPLIT_PATTERN=Pattern.compile("^(?:(?:[a-zA-Z][a-z]+)|(?:[0-9]+(?:[\\.,][0-9]+)*)|(?:U\\.S\\.?))(?:-(?:(?:[0-9]+)|(?:U\\.\\S\\.?)|(?:[a-zA-Z][a-z]+)))+$"); //This can be compounds of the form 'German-Swedish relations', it can also contain dots, as in 'U.S.-German relations', it can also contain numbers (that can have dots or commas): '12-year-old', '200,000-euro', but in this case I think the number can only be the very first token
    static final Pattern DONT_SPLIT_PATTERN=Pattern.compile("^(?:(?:[Aa]nti)|(?:[Cc]o)|(?:[Cc]ounter)|(?:[Cc]ross)|(?:[Cc]yber)|(?:[Dd]e)|(?:[Ee])|(?:[Ee]x)|(?:[Ii]nter)|(?:[Mm][ia]cro)|(?:[Mm]ega)|(?:[Mm]id)|(?:[Mm]ulti)|(?:[Nn]on)|(?:[Pp]ost)|(?:[Pp]r[eo])|(?:[Rr]e)|(?:[Vv]ice))-");//||[Dd]e|[Ee]x|[Vv]ice)-");
    static final Pattern HYPHEN_PATTERN=Pattern.compile("\\-");
	private String[] handleHyphens(String hwString) {
		if(POTENTIAL_SPLIT_PATTERN.matcher(hwString).matches() && !DONT_SPLIT_PATTERN.matcher(hwString).find()){
			return HYPHEN_PATTERN.split(hwString);
		} else {
			return new String[]{hwString};
		}
	}

	public List<List<String>> hasWordToStringSentences(List<List<HasWord>> sentences){
		List<List<String>> l=new ArrayList<List<String>>();
		for(List<HasWord> sent:sentences){
			List<String> s=hasWordToString(sent);
			if(doNormalize){
				String[] tokens=s.toArray(new String[s.size()]);
				String[] norm=TokenNormalizer.getNormalized(tokens);
				List<String> asList=Arrays.asList(norm);
				l.add(asList);
			} else {
				l.add(s);
			}
		}
		return l;
	}
	
}
