package ims.gw.tossduplicates;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;

import org.kohsuke.args4j.CmdLineException;
import org.kohsuke.args4j.CmdLineParser;
import org.kohsuke.args4j.Option;

public class Options {

	public static enum Format { OneSentencePerLine, CoNLL09 }
	
	@Option(name="-inputFormat",usage="Input format (default CoNLL09)")
	public Format inputFormat=Format.CoNLL09;
	@Option(name="-outputFormat",usage="Output format (default CoNLL09)")
	public Format outputFormat=Format.CoNLL09;
	
	@Option(name="-input",aliases="--inputFile",usage="Input file when doing one file only")
	public File input;
	@Option(name="-output",aliases="--outputFile",usage="Output file when doing when file only")
	public File output;
	@Option(name="-inEnc",aliases="--inputEncoding",usage="File encoding on input")
	public String inputEnc="UTF8";
	@Option(name="-outEnc",aliases="--outputEncoding",usage="File encoding on output")
	public String outputEnc="UTF8";
	
	@Option(name="-inputFiles",usage="Reads a list of files to process from STDIN (one file per line)")
	private boolean readFileNamesFromSTDIN=false;
	
	public File[] inputFiles;
	@Option(name="-outputSf",aliases="--outputFilesSuffix",usage="The suffix of the output files when using a list of input files (default '.nodups'")
	public String outputFilesSuffix=".nodups.gz";
	

	@Option(name="-help",aliases="--help",usage="Display help")
	public boolean help=false;

	
	private void setInputFiles() throws IOException{
		BufferedReader br=new BufferedReader(new InputStreamReader(System.in));
		List<String> q=new ArrayList<String>();
		String line;
		while((line=br.readLine())!=null)
			q.add(line);
		String[] f=q.toArray(new String[q.size()]);
		inputFiles=new File[f.length];
		for(int i=0;i<f.length;++i){
			inputFiles[i]=new File(f[i]);
			if(!inputFiles[i].exists())
				die("Input file "+inputFiles[i].getAbsolutePath()+" does not exist");
			if(!inputFiles[i].canRead())
				die("Cannot read input file "+inputFiles[i].getAbsolutePath());
		}
		System.out.println("Read "+inputFiles.length+" filenames from STDIN.");
	}
	
	public Options(String[] args) throws IOException{
		CmdLineParser clp=new CmdLineParser(this);
		try {
			clp.parseArgument(args);
		} catch (CmdLineException e) {
			e.printStackTrace();
			System.err.println("Failed to parse cmd line, aborting.");
			System.err.println();
			System.err.println("Valid options are:");
			printUsage(System.err);
			System.exit(1);
		}
		if(help){
			printUsage(System.out);
			System.exit(1);
		}
		if(readFileNamesFromSTDIN)
			setInputFiles();
	}
	
	
	private Options(){}
	public static void printUsage(OutputStream out){
		CmdLineParser clp=new CmdLineParser(new Options());
		clp.printUsage(out);
	}

	
	
	public static void die(String string) {
		System.err.println(string);
		System.err.println();
		System.err.println("Usage");
		printUsage(System.err);
		System.exit(1);
	}

}
