package ims.nbest;

import is2.data.SentenceData09;
import is2.io.CONLLReader09;
import is2.io.D8NBestCONLLReader09;
import is2.io.D8NBestCONLLReader09.NBestSentenceData09;
import is2.util.ParserEvaluator;

import java.io.File;
import java.io.PrintStream;

public class NBestOracle {

	private File out;
	private File gold;
	private boolean oracleLabeled=true;
	private int maxN=9999999;
	
	private int totalTok;
	private int corrLblTok;
	private int corrULblTok;
	
	private int totalSen;
	private int corrLblSen;
	private int corrULblSen;
	
	private int NtotalTok;
	private int NcorrLblTok;
	private int NcorrULblTok;
	
	private int NtotalSen;
	private int NcorrLblSen;
	private int NcorrULblSen;
	
	private int NOracles;
	
	private int[] highestOracleRank;
	private int[] NhighestOracleRank;
	
	NBestOracle(String[] args){
		parseArgs(args);
	}
	
	private void evaluate() {
		CONLLReader09 goldReader       =new CONLLReader09(gold.toString(), -1);
		D8NBestCONLLReader09 predReader=new D8NBestCONLLReader09(out.toString(), -1);
		SentenceData09 g=goldReader.getNext();
		NBestSentenceData09 p=predReader.getNextCoNLL09();
		int senCount=1;
		while(g !=null){
			if(!same(g,p))
				throw new Error("not same sentence: number "+senCount);
			int[]    goldHeads=g.heads;
			String[] goldLabels=g.labels;
			double bestAs=-1d;
			double[] ass=new double[Math.min(p.parses.length,maxN)];
			for(int i=0;i<ass.length;++i){
				double as=getAS(goldHeads,goldLabels,p.parses[i]);
				ass[i]=as;
				bestAs=Math.max(as, bestAs);
			}
			int oracleInd=-1;
			int oracleCount=0;
			boolean addToN=ass.length>1;
			for(int i=0;i<ass.length;++i){
				if(ass[i]==bestAs){
					if(oracleInd==-1){
						oracleInd=i;
					}
					oracleCount++;
				}
			}
			
			if(addToN)
				NOracles+=oracleCount;
			int tokens=goldHeads.length-1;
			int corru=0;
			int corrl=0;
			LabeledParse oracle=p.parses[oracleInd];
			for(int i=1;i<goldHeads.length;++i){
				if(goldHeads[i]==oracle.heads[i]){
					corru++;
					if(goldLabels[i].equals(oracle.labels[i]))
						corrl++;
				}
			}
			
			boolean exactMatchU=corru==tokens;
			boolean exactMatchL=corrl==tokens;
			
			totalTok+=tokens;
			corrLblTok+=corrl;
			corrULblTok+=corru;
			
			totalSen++;
			if(exactMatchU) corrULblSen++;
			if(exactMatchL) corrLblSen++;

			highestOracleRank=addHighestOracle(oracleInd,highestOracleRank);
			
			if(addToN){
				NtotalTok+=tokens;
				NcorrLblTok+=corrl;
				NcorrULblTok+=corru;
				NtotalSen++;
				if(exactMatchU) NcorrULblSen++;
				if(exactMatchL) NcorrLblSen++;
				NhighestOracleRank=addHighestOracle(oracleInd,NhighestOracleRank);
			}
			
			
			
			g=goldReader.getNext();
			p=predReader.getNextCoNLL09();
			senCount++;
		}
	}


	private int[] addHighestOracle(int i, int[] rankarr) {
		if(rankarr==null){
			int[] r=new int[i+1];
			r[i]++;
			return r;
		} else if(i<rankarr.length){
			rankarr[i]++;
			return rankarr;
		} else {
			int[] r=new int[i+1];
			System.arraycopy(rankarr, 0, r, 0, rankarr.length);
			r[i]++;
			return r;
		}
	}




	private double getAS(int[] goldHeads, String[] goldLabels,LabeledParse predParse) {
		int total=goldHeads.length-1;
		int corr=0;
		for(int i=1;i<goldHeads.length;++i)
			if(goldHeads[i]==predParse.heads[i] && (!oracleLabeled || goldLabels[i].equals(predParse.labels[i])))
				corr++;
		return (double)corr/total;
	}




	private boolean same(SentenceData09 g, NBestSentenceData09 p) {
		if(g.forms.length!=p.forms.length)
			return false;
		for(int i=0;i<g.forms.length;++i){
			if(!g.forms[i].equals(p.forms[i]))
				return false;
		}
		return true;
	}

	private void parseArgs(String[] args) {
		for(int ai=0;ai<args.length;++ai){
			if(args[ai].equals("-out")){
				out=new File(args[++ai]);
			} else if(args[ai].equals("-gold")){
				gold=new File(args[++ai]);
			} else if(args[ai].equals("-maxN")){
				maxN=Integer.parseInt(args[++ai]);
			} else if(args[ai].equals("-ulbl")){
				oracleLabeled=false;
			}
		}
	}

	private void oneBestEval() {
		CONLLReader09 goldReader=new CONLLReader09(gold.toString(), -1);
		CONLLReader09 predReader=new D8NBestCONLLReader09(out.toString(), -1);
		ParserEvaluator.evaluate(goldReader, predReader);
	}

	private void printEval() {
		System.out.println("=====   ALL   =====");
		System.out.printf("  UAS   100.0 * %-6d / %-6d  = %5.3f\n",corrULblTok,totalTok,100.0*corrULblTok/totalTok);
		System.out.printf("  LAS   100.0 * %-6d / %-6d  = %5.3f\n",corrLblTok,totalTok,100.0*corrLblTok/totalTok);
		System.out.printf("  UEM   100.0 * %-6d / %-6d  = %5.3f\n",corrULblSen,totalSen,100.0*corrULblSen/totalSen);
		System.out.printf("  LEM   100.0 * %-6d / %-6d  = %5.3f\n",corrLblSen,totalSen,100.0*corrLblSen/totalSen);
		System.out.println();
		System.out.println("=====  Ranks (all)  =====");
		printRankArray(highestOracleRank,System.out);
		System.out.println();
		System.out.println();
		System.out.println("=====   N>1   =====");
		System.out.printf("  UAS   100.0 * %-6d / %-6d  = %5.3f\n",NcorrULblTok,NtotalTok,100.0*NcorrULblTok/NtotalTok);
		System.out.printf("  LAS   100.0 * %-6d / %-6d  = %5.3f\n",NcorrLblTok,NtotalTok,100.0*NcorrLblTok/NtotalTok);
		System.out.printf("  UEM   100.0 * %-6d / %-6d  = %5.3f\n",NcorrULblSen,NtotalSen,100.0*NcorrULblSen/NtotalSen);
		System.out.printf("  LEM   100.0 * %-6d / %-6d  = %5.3f\n",NcorrLblSen,NtotalSen,100.0*NcorrLblSen/NtotalSen);
		System.out.println();
		System.out.println("=====   Ranks (N>1)  =====");
		printRankArray(NhighestOracleRank,System.out);
		System.out.println();
		System.out.println("NOracles: "+NOracles);
		System.out.println();
		
	}
	
	private void printRankArray(int[] rank, PrintStream out) {
		for(int i=0;i<rank.length;++i)
			if(rank[i]!=0)
				out.printf("%-5d   %d\n", i,rank[i]);
	}

	public static void main(String[] args){
		NBestOracle oracle=new NBestOracle(args);
		oracle.evaluate();
		oracle.printEval();
		System.out.println("One-best evaluation: ");
		oracle.oneBestEval();
	}

}
