package ims.normalization;

import is2.data.SentenceData09;
import is2.io.CONLLReader09;
import is2.io.CONLLWriter09;

import java.io.IOException;
import java.util.Arrays;
import java.util.regex.Pattern;

public class TokenNormalizer {

	public static class PairedTokenList {
		public final String[] original;
		public final String[] normalized;
		PairedTokenList(String[] original,String[] normalized){
			this.original=original;
			this.normalized=normalized;
		}
	}
	
	
	static final String URL_PLACEHOLDER="<<-URL->>";
	
	public static void normalizeURLs(String[] in){
		for(int i=0;i<in.length;++i){
			if(in[i].startsWith("http://") ||
					in[i].startsWith("https://") ||
					in[i].startsWith("ftp://") ||
					in[i].startsWith("www.") ||
					in[i].endsWith(".htm") ||
					in[i].endsWith(".html") ||
					in[i].endsWith(".php")
					){
				System.out.println(in[i]);
				in[i]=URL_PLACEHOLDER;
			}
		}
	}
	
	static final Pattern NUMBER_PATTERN=Pattern.compile("\\d");
	
	public static void normalizeNumbers(String[] in){
		for(int i=0;i<in.length;++i)
			in[i]=NUMBER_PATTERN.matcher(in[i]).replaceAll("0");
	}
	
	public static String[] getNormalized(String[] tokens){
		String[] r=Arrays.copyOf(tokens, tokens.length);
		normalizeURLs(r);
		normalizeNumbers(r);
		return r;
	}
	
	public static PairedTokenList getNormalizedPairedList(String[] tokens){
		String[] norm=getNormalized(tokens);
		return new PairedTokenList(tokens,norm);
	}
	
	static final Pattern WHITESPACE_PATTERN=Pattern.compile("\\s+");
	
	public static void main(String[] args) throws IOException{
		String input=args[0];
		String output=args[1];
		CONLLReader09 reader=new CONLLReader09();
		CONLLWriter09 writer=new CONLLWriter09(output);
		reader.startReading(input);
		SentenceData09 inst;
		while(true){
			inst=reader.getNext();
			if(inst==null)
				break;
			inst.forms=getNormalized(inst.forms);
			writer.write(inst, true);
		}
		writer.finishWriting();
	}
}
