package ims.util;

public class MinHeap<T extends HasScore> {
	private T[] arr;
	int size=0;
	
	public MinHeap(T[] arr){
		this.arr=arr;
	}
	public void insert(T t){
		if(size==arr.length){
			popAndInsert(t);
			//Pop and insert
		} else { //Toss
			arr[size++]=t;
			if(size==arr.length)
				heapify();
		}
	}
	public boolean offer(float d){
		if(size==arr.length){
			if(d>arr[0].score()){
				return true; //The offered element is greater, then we want it
			} else {
				return false; //The offered element is less, then we don't want it
			}
		} else {
			return true;
		}
	}
	public int size(){
		return size;
	}
	public void clear(){
		size=0;
	}
	public int emptyReverseInto(T[] t){
		//Note that if we have only tossed things and didn't hit the max yet, it is not heapified at this point. Then we need to heapify it ourselves
		int itemCount=0;
		if(size!=arr.length)
			heapify();
		for(int i=size-1;i>=0;--i,itemCount++)
			t[i]=pop();
		return itemCount;
	}
	private T pop(){
		T ret=arr[0];
		arr[0]=arr[--size];
		heapify(0);
		return ret;
	}
	private void popAndInsert(T t){
		arr[0]=t;
		heapify(0);
	}
	private void heapify(){
		for(int i=parent(size-1);i>=0;--i)
			heapify(i);
	}
	private void heapify(int i){
		int l=left(i);
		int r=right(i);
		int smallest=i;
		if(l<size && arr[l].score()<arr[i].score())
			smallest=l;
		if(r<size && arr[r].score()<arr[smallest].score())
			smallest=r;
		if(smallest!=i){
			swap(i,smallest);
			heapify(smallest);
		}
	}
	private int left(int i)		{	return 2*i+1;}
	private int right(int i)	{	return 2*i+2;}
	private int parent(int i)	{	return (i-1)/2;}
	private void swap(int a,int b){
		T tmp=arr[a];
		arr[a]=arr[b];
		arr[b]=tmp;
	}
	
	public boolean hasRoomLeft(){
		return size<arr.length;
	}
	public T peek(){
		if(hasRoomLeft())
			heapify();
		return arr[0];
	}
}
