package is2.parser;

import ims.util.HasScore;
import ims.util.MinHeap;
import is2.data.DataFES;
import is2.data.Parse;
import is2.parser.ParallelRearrangeNBestD8.RearrangeJob;
import is2.parser.ParallelRearrangeNBestD8.SyncJobQueue;
import is2.parserR2.Parser;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;

public class D8NBestByRearranging {

	/**
	 * This is the beam used given one specific seed tree
	 */
	public static int treeBeam=20;
	/**
	 * This is how many trees are put back as seeds out of all trees
	 * from one round over all seeds
	 */
	public static int seedBeam=12;
	/**
	 * All trees that are kept need to improve the score by this much.	
	 */
	static float scoreTh=-0.5f;
	/**
	 * Dummy parameters that constrains the number of steps maximal.
	 */
	static int maxRounds=50;

	
	static int brokeRounds=0;
	static int brokeEmptySeeds=0;
	static int brokeFull=0;
	static int brokeNonFull=0;
	
	public static List<D8Parse> rearrangeNBest(short[] pos, short[] heads_,short[] labs_, DataFES x, int n) throws InterruptedException, ExecutionException {
		float startScore=Extractor.encode3(pos, heads_, labs_, x);
		D8Parse startParse=new D8Parse(heads_,labs_,startScore);
		startParse.signature();
		
		float lastBest=Float.NEGATIVE_INFINITY;
		float bestNBest=startScore;
		
		gnu.trove.THashSet<CharSequence> explored = new gnu.trove.THashSet<CharSequence>();
		gnu.trove.THashSet<CharSequence> addedToOut = new gnu.trove.THashSet<CharSequence>();
		List<D8Parse> seeds=new ArrayList<D8Parse>();
		seeds.add(startParse);
		MinHeap<D8Parse> out=new MinHeap<D8Parse>(new D8Parse[n]);
		out.insert(startParse);
		
		int lastOutSize=-1;
		float lastOutWorstScore=Float.NEGATIVE_INFINITY;
		
		for(int round=0;;++round){
			if(round==maxRounds){
				brokeRounds++;
				break;
			}
			if(seeds.size()==0){
				brokeEmptySeeds++;
				break;
			}
			MinHeap<NewTree> newTrees=new MinHeap<NewTree>(new NewTree[n]);
			List<D8Parse> newSeeds=new ArrayList<D8Parse>();
			gnu.trove.THashSet<CharSequence> addedToNewTrees=new gnu.trove.THashSet<CharSequence>();
			for(D8Parse seed:seeds){
				String sig=seed.signature();
				if(explored.contains(sig))
					continue;
				else 
					explored.add(sig);
				
				short[] heads=seed.heads;
				boolean[][] transClosure=new boolean[heads.length][heads.length];
				for(int i = 1, l1=1; i < heads.length; i++,l1=i)  
					while((l1= heads[l1]) != -1) transClosure[l1][i] = true;
				
				List<RearrangeJob> jobs=new ArrayList<RearrangeJob>();
				for(short ch = 1; ch < heads.length; ch++) {
					for(short pa = 0; pa < heads.length; pa++) {
						if(ch == pa || pa == heads[ch] || transClosure[ch][pa]) 
							continue;
						RearrangeJob j=new RearrangeJob(pa, ch);
						jobs.add(j);
					}
				}
				int threads =Math.min(Parser.THREADS,jobs.size());
				
				SyncJobQueue jq=new SyncJobQueue(new ArrayList<RearrangeJob>(jobs));
				List<Callable<Void>> workers=new ArrayList<Callable<Void>>();
				for(int i=0;i<threads;++i)
					workers.add(new ParallelRearrangeNBestD8(pos,Arrays.copyOf(heads,heads.length),Arrays.copyOf(seed.labels, seed.labels.length),x,jq));
				for(Future<Void> f:Decoder.executerService.invokeAll(workers))
					f.get();
				
				//Now take the beam best trees out of the jobs list
				MinHeap<Move> h=new MinHeap<Move>(new Move[treeBeam]);
				for(RearrangeJob rj:jobs){
					for(int i=0;i<rj.outScores.length;++i){
						Move m=new Move(rj,i);
						if(addedToNewTrees.contains(m.sig(seed)))
							continue;
						if(h.offer(rj.outScores[i]))
							h.insert(m);
					}
				}
				
				Move[] top=new Move[h.size()];
				h.emptyReverseInto(top);
				for(Move m:top){
					String mSig=m.sig(seed);
					if(m.score()<seed.f1+scoreTh || !newTrees.offer(m.score()))
						break;
					if(addedToNewTrees.contains(mSig))
						continue;
					newTrees.insert(new NewTree(m,seed));
					addedToNewTrees.add(mSig);
				}
			}
			NewTree[] top=new NewTree[newTrees.size()];
			newTrees.emptyReverseInto(top);
			lastBest=bestNBest;
			bestNBest=(top.length>0?Math.max(bestNBest,top[0].score()):bestNBest);
			boolean bestNBestImproved=bestNBest>lastBest;
			int q=0;
			for(NewTree nt:top){
				D8Parse p=nt.toD8Parse();
				boolean toOut=out.offer(nt.score());
				boolean toSeeds=q++<seedBeam;
				String sig=p.signature();
				if(toSeeds && !explored.contains(sig))
					newSeeds.add(p);
				if(toOut && !addedToOut.contains(sig)){
					out.insert(p);
					addedToOut.add(sig);
				}
				if(!toSeeds && !toOut)
					break;
			}
			seeds=newSeeds;
			
			//Some stopping criteria
			if(!out.hasRoomLeft()){ //We have a 1000 now.
				float worstScore=out.peek().score();
				if(worstScore==lastOutWorstScore && !bestNBestImproved){
					brokeFull++;
					break;
				}
				lastOutWorstScore=worstScore;
			} else if(out.size()==lastOutSize && !bestNBestImproved){
				brokeNonFull++;
				break;
			}
			lastOutSize=out.size();
		}
		D8Parse[] o=new D8Parse[out.size()];
		out.emptyReverseInto(o);
		return Arrays.asList(o);
	}
	static class Move implements HasScore{
		private final RearrangeJob rj;
		private final int solIdx;
		Move(RearrangeJob j,int mIdx){
			this.rj=j;
			this.solIdx=mIdx;
		}
		public float getTreeScore(){
			return rj.outScores[solIdx];
		}
		@Override
		public float score() {
			return getTreeScore();
		}
		String sig=null;
		public String sig(D8Parse seed){
			if(sig!=null)
				return sig;
			StringBuilder sb=new StringBuilder(seed.heads.length*2);
			for(int i=0;i<seed.heads.length;++i)
				if(i==rj.ch)
					sb.append((char)rj.pa).append((char) rj.outLabs[solIdx]);
				else
					sb.append((char)seed.heads[i]).append((char) seed.labels[i]);
			sig=sb.toString();
			return sig;
		}
	}
	static class NewTree implements HasScore{
		private final Move move;
		private final D8Parse seed;
		NewTree(Move m,D8Parse seed){
			this.move=m;
			this.seed=seed;
		}
		public D8Parse toD8Parse(){
			D8Parse p=new D8Parse(Arrays.copyOf(seed.heads, seed.heads.length),Arrays.copyOf(seed.labels, seed.labels.length),move.score());
			p.heads[move.rj.ch]=move.rj.pa;
			p.labels[move.rj.ch]=move.rj.outLabs[move.solIdx];
			return p;
		}
		@Override
		public float score() {
			return move.score();
		}
	}
	
	static class D8Parse extends Parse implements HasScore {
		public D8Parse(short[] heads,short[] labels, float score){
			this.heads=heads;
			this.labels=labels;
			this.f1=score;
		}
		@Override
		public float score() {
			return (float) f1;
		}
	}
}
