package is2.parser;

import java.util.List;

import ims.util.Pair;
import is2.data.Instances;
import is2.data.Parse;
import is2.data.PipeGen;
import is2.data.SentenceData09;
import is2.io.CONLLReader09;
import is2.io.D8NBestCONLLReader09;
import is2.io.D8NBestCONLLWriter09;
import is2.util.DB;
import is2.util.OptionsSuper;
import is2.util.ParserEvaluator;

public class ParserNBest extends Parser {
	private static int nBest;
	private static boolean skipNBest=false;
	
	int totalTreesOut=0;
	int minTreesOut=99999;
	int maxTreesOut=0;
	int sentences=0;
	int singleTreeOutSentences=0;
	int nonNTreesOut=0;
	
	void outNBest(OptionsSuper options,Pipe pipe, ParametersFloat params,boolean maxInfo,int n) throws Exception {
		
		DB.println("NBest out params:");
		DB.println("nBest: "+nBest+", treeBeam: "+D8NBestByRearranging.treeBeam+", seedBeam: "+D8NBestByRearranging.seedBeam+", nBestTh: "+D8NBestByRearranging.scoreTh);
		
		long start = System.currentTimeMillis();
		
		CONLLReader09 depReader = new CONLLReader09(options.testfile, options.formatTask);
		D8NBestCONLLWriter09 depWriter = new D8NBestCONLLWriter09(options.outfile, options.formatTask);

		Extractor.initFeatures();

		Extractor.initFeatures();

//		int cnt = 0;
		int del=0;
		long last = System.currentTimeMillis();

		if (maxInfo) System.out.println("\nParsing Information ");
		if (maxInfo) System.out.println("------------------- ");

//		if (maxInfo && !options.decodeProjective) System.out.println(""+Decoder.getInfo());

		System.out.print("Processing Sentence: ");

		while(true) {

			Instances is = new Instances();
			is.init(1, new MFO(),options.formatTask);

			SentenceData09 instance = pipe.nextInstance(is, depReader);
			if (instance==null) break;
			sentences++;

//			SentenceData09 i09 = this.parse(instance,params, labelOnly,options);
//			depWriter.write(i09);
			Pair<SentenceData09,List<? extends Parse>> p=this.parseNBest(instance, params, options, n);
			List<? extends Parse> parses=p.getRight();
			totalTreesOut+=parses.size();
			minTreesOut=Math.min(minTreesOut, parses.size());
			maxTreesOut=Math.max(maxTreesOut, parses.size());
			if(parses.size()!=n)
				nonNTreesOut++;
			if(parses.size()==1)
				singleTreeOutSentences++;
			depWriter.write(p.getLeft(),false,true);
			if(skipNBest)
				depWriter.newLine();
			else
				depWriter.writeNBest(parses,_types);
			del=PipeGen.outValue(sentences, del,last);
			String e="       "+p.getRight().size()+"   ";
			System.out.print(e);
			del+=e.length();

		}
		System.out.println();
		//pipe.close();
		depWriter.finishWriting();
		long end = System.currentTimeMillis();
		//		DB.println("errors "+error);
		if (maxInfo) System.out.println("Used time " + (end-start));
		if (maxInfo) System.out.println("forms count "+Instances.m_count+" unkown "+Instances.m_unkown);
		if (maxInfo) System.out.println(String.format("Sent: %d | Trees: %d (avg/sent: %4.2f) (min: %d, max: %d)",sentences,totalTreesOut,((float) totalTreesOut/sentences),minTreesOut,maxTreesOut));
		if (maxInfo) System.out.println(String.format("Single tree sentences: %d | Not n trees out: %d |Multi-tree sentences: %d (avg/sent: %4.2f)",singleTreeOutSentences,nonNTreesOut,sentences-singleTreeOutSentences,(float)(totalTreesOut-singleTreeOutSentences)/(sentences-singleTreeOutSentences)));
		if (maxInfo) System.out.println("Rearranger stopping: ");
		if (maxInfo) System.out.println(" Empty seed list:  "+D8NBestByRearranging.brokeEmptySeeds);
		if (maxInfo) System.out.println(" Full no increase: "+D8NBestByRearranging.brokeFull);
		if (maxInfo) System.out.println(" No growth:        "+D8NBestByRearranging.brokeNonFull);
		if (maxInfo) System.out.println(" Max rounds:       "+D8NBestByRearranging.brokeRounds);
	}
	
	public Pair<SentenceData09, List<? extends Parse>> parseNBest(SentenceData09 instance,ParametersFloat params,OptionsSuper options,int n) throws Exception {
		is = new Instances();
		is.init(1, new MFO(),options.formatTask);
		new CONLLReader09().insert(is, instance); 

		SentenceData09 i09 = new SentenceData09(instance);
		i09.createSemantic(instance);

//		pipe.fillVector(params.getFV(), is,0,null,pipe.cl);//cnt-1
		d2 = pipe.fillVector(params.getFV(), is,0,null,pipe.cl);//cnt-1
		
		List<? extends Parse> nBest=Decoder.decodeNBest(is.pposs[0],d2,options.decodeProjective, !Decoder.TRAINING,n); //cnt-1
		Parse d=nBest.get(0);
		for(int j = 0; j < instance.forms.length-1; j++) {
			i09.plabels[j] = _types[d.labels[j+1]];
			i09.pheads[j] = d.heads[j+1];
		}
//		if (parses.size()>NBest) parses = parses.subList(0,NBest);
		if(nBest.size()>n)
			nBest=nBest.subList(0, n);
		return new Pair<SentenceData09,List<? extends Parse>>(i09,nBest);
	}
	
	public static void main(String[] args) throws Exception{
		long start = System.currentTimeMillis();
		OptionsSuper options = new Options(args);
		
		nBest=options.best;
		
		parseNBestSpecificOptions(args);
		
		Runtime runtime = Runtime.getRuntime();
		Parser.THREADS = runtime.availableProcessors();
		if (options.cores<Parser.THREADS&&options.cores>0) Parser.THREADS =options.cores;		
		DB.println("Found " + runtime.availableProcessors()+" cores use "+Parser.THREADS);

		
		if (options.test) {

			ParserNBest p = new ParserNBest();
			p.options=options;


			p. pipe = new Pipe(options);
			p. params = new ParametersFloat(0);  // total should be zero and the parameters are later read

			// load the model

			p.readModel(options, p.pipe, p.params);

			DB.println("label only? "+options.label);
			DB.println("n-best"+options.best);
			p.outNBest(options, p.pipe, p.params, true,nBest);
		}
		System.out.println();

		if (options.eval) {
			System.out.println("\nEVALUATION PERFORMANCE:");
			CONLLReader09 goldReader=new CONLLReader09(options.goldfile, -1);
			CONLLReader09 predReader=skipNBest?new CONLLReader09(options.outfile, -1):new D8NBestCONLLReader09(options.outfile, -1);
			ParserEvaluator.evaluate(goldReader, predReader);
		}

		long end = System.currentTimeMillis();
		System.out.println("used time "+((float)((end-start)/100)/10));

		Decoder.executerService.shutdown();		
		Pipe.executerService.shutdown();
		System.out.println("end.");
	}

	
	
	private static void parseNBestSpecificOptions(String[] args) {
		for(int ai=0;ai<args.length;++ai){
			if(args[ai].equals("--treeBeam")){
				D8NBestByRearranging.treeBeam=Integer.parseInt(args[++ai]);
			} else if(args[ai].equals("--seedBeam")){
				D8NBestByRearranging.seedBeam=Integer.parseInt(args[++ai]);
			} else if(args[ai].equals("--nBestTh")){
				D8NBestByRearranging.scoreTh=Float.parseFloat(args[++ai]);
			} else if(args[ai].equals("--nBest")){
				nBest=Integer.parseInt(args[++ai]);
			} else if(args[ai].equals("--skipNBest")){
				skipNBest=true;
			}
//			} else if(args[ai].equals("--seedLoopTh")){
//				D8NBestByRearranging.seedLoopTh=Float.parseFloat(args[++ai]);
//			}
		}
	}
}
