/*
 *  ICARUS -  Interactive platform for Corpus Analysis and Research tools, University of Stuttgart
 *  Copyright (C) 2012-2013 Markus Gärtner and Gregor Thiele
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses.
 *
 * $Revision: 229 $
 * $Date: 2014-02-20 15:53:33 +0100 (Do, 20 Feb 2014) $
 * $URL: https://subversion.assembla.com/svn/icarusplatform/trunk/Icarus/plugins/de.ims.icarus.plugins.coref/source/de/ims/icarus/language/coref/CorefListMember.java $
 *
 * $LastChangedDate: 2014-02-20 15:53:33 +0100 (Do, 20 Feb 2014) $
 * $LastChangedRevision: 229 $
 * $LastChangedBy: mcgaerty $
 */
package de.ims.icarus.language.coref;

import java.util.ArrayList;
import java.util.List;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.EventListenerList;

import de.ims.icarus.util.data.DataList;
import de.ims.icarus.util.mem.HeapMember;
import de.ims.icarus.util.mem.Link;

/**
 * @author Markus Gärtner
 * @version $Id: CorefListMember.java 229 2014-02-20 14:53:33Z mcgaerty $
 *
 */
@HeapMember
public abstract class CorefListMember<T extends Object> extends CorefMember implements DataList<T> {

	protected EventListenerList listenerList = new EventListenerList();

	@Link
	protected List<T> items;

	protected CorefListMember() {
		// no-op
	}

	/**
	 * @see de.ims.icarus.util.data.DataList#size()
	 */
	@Override
	public int size() {
		return items==null ? 0 : items.size();
	}

	/**
	 * @see de.ims.icarus.util.data.DataList#get(int)
	 */
	@Override
	public T get(int index) {
		return items==null ? null : items.get(index);
	}

	public void add(T data) {
		if(data==null)
			throw new NullPointerException("Invalid data"); //$NON-NLS-1$

		if(items==null) {
			items = new ArrayList<>();
		}

		items.add(data);

		fireChangeEvent();
	}

	public void free() {
		if(items!=null) {
			items.clear();
		}

		items = null;
		properties = null;
		fireChangeEvent();
	}

	/**
	 * @see de.ims.icarus.util.data.DataList#addChangeListener(javax.swing.event.ChangeListener)
	 */
	@Override
	public void addChangeListener(ChangeListener listener) {
		listenerList.add(ChangeListener.class, listener);
	}

	/**
	 * @see de.ims.icarus.util.data.DataList#removeChangeListener(javax.swing.event.ChangeListener)
	 */
	@Override
	public void removeChangeListener(ChangeListener listener) {
		listenerList.remove(ChangeListener.class, listener);
	}

	protected void fireChangeEvent() {
		Object[] listeners = listenerList.getListeners(ChangeListener.class);

		if(listeners==null || listeners.length==0) {
			return;
		}

		ChangeEvent evt = new ChangeEvent(this);

		for(Object listener : listeners) {
			((ChangeListener)listener).stateChanged(evt);
		}
	}
}
