/*
 *  ICARUS -  Interactive platform for Corpus Analysis and Research tools, University of Stuttgart
 *  Copyright (C) 2012-2013 Markus Gärtner and Gregor Thiele
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses.
 *
 * $Revision: 459 $
 * $Date: 2016-05-16 23:25:11 +0200 (Mo, 16 Mai 2016) $
 * $URL: https://subversion.assembla.com/svn/icarusplatform/trunk/Icarus/plugins/de.ims.icarus.plugins.coref/source/de/ims/icarus/plugins/coref/view/CoreferenceDocumentView.java $
 *
 * $LastChangedDate: 2016-05-16 23:25:11 +0200 (Mo, 16 Mai 2016) $ 
 * $LastChangedRevision: 459 $ 
 * $LastChangedBy: mcgaerty $
 */
package de.ims.icarus.plugins.coref.view;

import java.awt.BorderLayout;

import javax.swing.JComponent;

import de.ims.icarus.language.coref.DocumentData;
import de.ims.icarus.plugins.core.View;
import de.ims.icarus.util.mpi.Commands;
import de.ims.icarus.util.mpi.Message;
import de.ims.icarus.util.mpi.ResultMessage;

/**
 * @author Markus Gärtner
 * @version $Id: CoreferenceDocumentView.java 459 2016-05-16 21:25:11Z mcgaerty $
 *
 */
public class CoreferenceDocumentView extends View {
	
	private CoreferenceDocumentDataPresenter presenter;

	public CoreferenceDocumentView() {
		// no-op
	}

	/**
	 * @see de.ims.icarus.plugins.core.View#init(javax.swing.JComponent)
	 */
	@Override
	public void init(JComponent container) {
		
		container.setLayout(new BorderLayout());
		
		presenter = new CoreferenceDocumentDataPresenter();
		
		container.add(presenter.getPresentingComponent(), BorderLayout.CENTER);
	}

	@Override
	public void close() {
		presenter.close();
	}

	@Override
	public void reset() {
		presenter.clear();
	}

	@Override
	protected ResultMessage handleRequest(Message message) throws Exception {
		if(Commands.PRESENT.equals(message.getCommand())
				|| Commands.DISPLAY.equals(message.getCommand())) {
			Object data = message.getData();
			if(data instanceof DocumentData) {
				//focusView();
				selectViewTab();
				presenter.present(data, message.getOptions());
				return message.successResult(this, null);
			} else {
				return message.unsupportedDataResult(this);
			}
		} else if(Commands.CLEAR.equals(message.getCommand())) {
			reset();
			return message.successResult(this, null);
		} else {
			return message.unknownRequestResult(this);
		}
	}
}