/*
 *  ICARUS -  Interactive platform for Corpus Analysis and Research tools, University of Stuttgart
 *  Copyright (C) 2012-2013 Markus Gärtner and Gregor Thiele
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses.
 *
 * $Revision: 159 $
 * $Date: 2013-11-04 11:20:05 +0100 (Mo, 04 Nov 2013) $
 * $URL: https://subversion.assembla.com/svn/icarusplatform/trunk/Icarus/plugins/de.ims.icarus.plugins.search_tools/source/de/ims/icarus/plugins/search_tools/view/results/SearchResultListWrapper.java $
 *
 * $LastChangedDate: 2013-11-04 11:20:05 +0100 (Mo, 04 Nov 2013) $ 
 * $LastChangedRevision: 159 $ 
 * $LastChangedBy: mcgaerty $
 */
package de.ims.icarus.plugins.search_tools.view.results;

import javax.swing.event.ChangeListener;

import de.ims.icarus.search_tools.result.ResultEntry;
import de.ims.icarus.search_tools.result.SearchResult;
import de.ims.icarus.util.annotation.AnnotatedData;
import de.ims.icarus.util.annotation.AnnotationContainer;
import de.ims.icarus.util.data.ContentType;
import de.ims.icarus.util.data.DataList;


/**
 * @author Markus Gärtner
 * @version $Id: SearchResultListWrapper.java 159 2013-11-04 10:20:05Z mcgaerty $
 *
 */
public class SearchResultListWrapper<T extends Object> implements DataList<T>, AnnotationContainer {

	private final SearchResult searchResult;
	
	public SearchResultListWrapper(SearchResult searchResult) {
		if(searchResult==null)
			throw new NullPointerException("Invalid search result"); //$NON-NLS-1$
		
		this.searchResult = searchResult;
	}
	
	public SearchResult getSearchResult() {
		return searchResult;
	}

	/**
	 * @see de.ims.icarus.util.data.DataList#size()
	 */
	@Override
	public int size() {
		return searchResult.getTotalMatchCount();
	}

	/**
	 * @see de.ims.icarus.util.data.DataList#get(int)
	 */
	@SuppressWarnings("unchecked")
	@Override
	public T get(int index) {
		ResultEntry entry = searchResult.getRawEntry(index);
		AnnotatedData annotatedData = searchResult.getAnnotatedEntry(entry);
		
		return (T) (annotatedData==null ? searchResult.getPlainEntry(entry) : annotatedData);
	}
	
	public Object getPlain(int index) {
		return searchResult.getEntry(index);
	}
	
	public ResultEntry getRaw(int index) {
		return searchResult.getRawEntry(index);
	}

	/**
	 * @see de.ims.icarus.util.data.DataList#getContentType()
	 */
	@Override
	public ContentType getContentType() {
		return searchResult.getContentType();
	}

	/**
	 * @see de.ims.icarus.util.data.DataList#addChangeListener(javax.swing.event.ChangeListener)
	 */
	@Override
	public void addChangeListener(ChangeListener listener) {
		// no-op
	}

	/**
	 * @see de.ims.icarus.util.data.DataList#removeChangeListener(javax.swing.event.ChangeListener)
	 */
	@Override
	public void removeChangeListener(ChangeListener listener) {
		// no-op
	}

	/**
	 * @see de.ims.icarus.util.annotation.AnnotationContainer#getAnnotationType()
	 */
	@Override
	public ContentType getAnnotationType() {
		return searchResult.getAnnotationType();
	}

}
