/*
 *  ICARUS -  Interactive platform for Corpus Analysis and Research tools, University of Stuttgart
 *  Copyright (C) 2012-2013 Markus Gärtner and Gregor Thiele
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses.
 *
 * $Revision: 464 $
 * $Date: 2016-08-02 15:35:21 +0200 (Di, 02 Aug 2016) $
 * $URL: https://subversion.assembla.com/svn/icarusplatform/trunk/Icarus/plugins/de.ims.icarus.plugins.search_tools/source/de/ims/icarus/search_tools/SearchMode.java $
 *
 * $LastChangedDate: 2016-08-02 15:35:21 +0200 (Di, 02 Aug 2016) $
 * $LastChangedRevision: 464 $
 * $LastChangedBy: mcgaerty $
 */
package de.ims.icarus.search_tools;

import javax.swing.Icon;

import de.ims.icarus.resources.ResourceManager;
import de.ims.icarus.search_tools.standard.DefaultSearchOperator;
import de.ims.icarus.util.id.Identity;


/**
 * @author Markus Gärtner
 * @version $Id: SearchMode.java 464 2016-08-02 13:35:21Z mcgaerty $
 *
 */
public enum SearchMode implements Identity {

	/**
	 * Every single hit encountered in a target graph
	 * should be cached and the graph reported as a whole.
	 * This effectively implies exhaustive searching!
	 *
	 * @deprecated The interaction of this search mode and the {@link DefaultSearchOperator#GROUPING grouping operator}
	 * causes undesired side effects in result set creation that are not easily fixed. Therefore searching will be restricted
	 * to only use the other 2 "basic" modes!
	 */
	@Deprecated
	HITS("hits", true), //$NON-NLS-1$

	/**
	 * Every single hit encountered in a target graph
	 * should be reported independently. This effectively
	 * implies exhaustive searching!
	 */
	INDEPENDENT_HITS("independentHits", true), //$NON-NLS-1$

	/**
	 * Only the first hit in a target graph should be reported.
	 * Further processing of that graph is not necessary.
	 */
	MATCHES("matches", false); //$NON-NLS-1$

	private final String key;
	private final boolean exhaustive;

	private SearchMode(String key, boolean exhaustive) {
		this.key = key;
		this.exhaustive = exhaustive;
	}

	/**
	 * @see de.ims.icarus.util.id.Identity#getId()
	 */
	@Override
	public String getId() {
		return getClass().getSimpleName();
	}

	/**
	 * @see de.ims.icarus.util.id.Identity#getName()
	 */
	@Override
	public String getName() {
		return ResourceManager.getInstance().get(
				"plugins.searchTools.searchMode."+key+".name"); //$NON-NLS-1$ //$NON-NLS-2$
	}

	/**
	 * @see de.ims.icarus.util.id.Identity#getDescription()
	 */
	@Override
	public String getDescription() {
		return ResourceManager.getInstance().get(
				"plugins.searchTools.searchMode."+key+".description"); //$NON-NLS-1$ //$NON-NLS-2$
	}

	/**
	 * @see de.ims.icarus.util.id.Identity#getIcon()
	 */
	@Override
	public Icon getIcon() {
		return null;
	}

	/**
	 * @see de.ims.icarus.util.id.Identity#getOwner()
	 */
	@Override
	public Object getOwner() {
		return this;
	}

	public boolean isExhaustive() {
		return exhaustive;
	}

	public static SearchMode[] supportedModes() {
		return new SearchMode[]{INDEPENDENT_HITS, MATCHES};
	}
}
